@@ -5,18 +5,18 @@ import os
 
 # has to be ordered tuple
 REPLACEMENTS=[
-	[' DREAL', ' float'],
-	[' SHORTREAL', ' float'],
-	[' LONGREAL', ' float'],
+	[' float64_t', ' float'],
+	[' float32_t', ' float'],
+	[' float128_t', ' float'],
 	[' double', ' float'],
-	[' INT', '  int'],
-	[' UINT', ' int'],
-	[' LONG', ' int'],
-	[' ULONG', ' int'],
-	[' BYTE', ' int'],
-	[' SHORT', ' int'],
-	[' WORD', ' int'],
-	[' CHAR', ' str'],
+	[' int32_t', '  int'],
+	[' uint32_t', ' int'],
+	[' int64_t', ' int'],
+	[' uint64_t', ' int'],
+	[' uint8_t', ' int'],
+	[' int16_t', ' int'],
+	[' uint16_t', ' int'],
+	[' char', ' str'],
 	['-> CFeatures', '-> Features'],
 ]
 
