@@ -57,6 +57,46 @@ void CSGObject::set_global_objects()
 	version=sg_version;
 }
 
+void CSGObject::set_io(CIO* new_io)
+{
+	SG_UNREF(sg_io);
+	sg_io=new_io;
+	SG_REF(sg_io);
+}
+
+CIO* CSGObject::get_io()
+{
+	SG_REF(sg_io);
+	return sg_io;
+}
+
+void CSGObject::set_parallel(CParallel* new_parallel)
+{
+	SG_UNREF(sg_parallel);
+	sg_parallel=new_parallel;
+	SG_REF(sg_parallel);
+}
+
+CParallel* CSGObject::get_parallel()
+{
+	SG_REF(sg_parallel);
+	return sg_parallel;
+}
+
+void CSGObject::set_version(CVersion* new_version)
+{
+	SG_UNREF(sg_version);
+	sg_version=new_version;
+	SG_REF(sg_version);
+}
+
+CVersion* CSGObject::get_version()
+{
+	SG_REF(sg_version);
+	return sg_version;
+}
+
+
 #ifdef HAVE_BOOST_SERIALIZATION
 std::string CSGObject::to_string() const
 {
