@@ -10,6 +10,15 @@
 
 #include "classifier/Classifier.h"
 
+
+#ifdef HAVE_BOOST_SERIALIZATION
+#include <boost/serialization/export.hpp>
+BOOST_CLASS_EXPORT_GUID(shogun::CClassifier, "CClassifier");
+//BOOST_SERIALIZATION_ASSUME_ABSTRACT(shogun::CClassifier);
+#endif //HAVE_BOOST_SERIALIZATION
+
+using namespace shogun;
+
 CClassifier::CClassifier() : CSGObject(), max_train_time(0), labels(NULL),
 	solver_type(ST_AUTO)
 {
@@ -19,25 +28,3 @@ CClassifier::~CClassifier()
 {
     SG_UNREF(labels);
 }
-
-CLabels* CClassifier::classify(CLabels* output)
-{
-	if (labels)
-	{
-		int32_t num=labels->get_num_labels();
-		ASSERT(num>0);
-
-		if (!output)
-		{
-			output=new CLabels(num);
-			SG_REF(output);
-		}
-
-		for (int32_t i=0; i<num; i++)
-			output->set_label(i, classify_example(i));
-
-		return output;
-	}
-
-	return NULL;
-}
