@@ -70,7 +70,8 @@ enum ESolverType
 	ST_CPLEX=1,
 	ST_GLPK=2,
 	ST_NEWTON=3,
-	ST_DIRECT=4
+	ST_DIRECT=4,
+	ST_ELASTICNET=5
 };
 
 /** @brief A generic classifier interface.
@@ -171,7 +172,13 @@ class CClassifier : public CSGObject
 		 * @param i index of label to get
 		 * @return value of label at index i
 		 */
-		virtual inline float64_t get_label(int32_t i) { return labels->get_label(i); }
+		virtual inline float64_t get_label(int32_t i)
+		{
+			if (!labels)
+				SG_ERROR("No Labels assigned\n");
+
+			return labels->get_label(i);
+		}
 
 		/** set maximum training time
 		 *
