@@ -12,6 +12,8 @@
 #include "features/Labels.h"
 #include "lib/Mathematics.h"
 
+using namespace shogun;
+
 CKernelPerceptron::CKernelPerceptron()
 : CKernelMachine()
 {
@@ -22,7 +24,7 @@ CKernelPerceptron::~CKernelPerceptron()
 {
 }
 
-bool CKernelPerceptron::train()
+bool CKernelPerceptron::train(CFeatures* data)
 {
 	ASSERT(labels);
 	//CLabels* train_labels=labels->get_int_labels(num_train_labels);
