@@ -16,6 +16,8 @@
 #include "classifier/svm/SVM.h"
 #include "lib/Cache.h"
 
+namespace shogun
+{
 /** @brief CplexSVM a SVM solver implementation based on cplex (unfinished). */
 class CCPLEXSVM : public CSVM
 {
@@ -23,10 +25,18 @@ class CCPLEXSVM : public CSVM
 		CCPLEXSVM();
 		virtual ~CCPLEXSVM();
 
-		virtual bool train();
+		/** train SVM classifier
+		 *
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
+		 */
+		virtual bool train(CFeatures* data=NULL);
 
 		virtual inline EClassifierType get_classifier_type() { return CT_CPLEXSVM; }
 };
-
+}
 #endif
 #endif
