@@ -15,7 +15,10 @@
 
 #include "lib/common.h"
 #include "classifier/svm/MultiClassSVM.h"
+#include "features/Features.h"
 
+namespace shogun
+{
 /** @brief Class GMNPSVM implements a one vs. rest MultiClass SVM.
  *
  * It uses CGMNPLib for training (in true multiclass-SVM fashion).
@@ -33,24 +36,44 @@ class CGMNPSVM : public CMultiClassSVM
 		 * @param lab labels
 		 */
 		CGMNPSVM(float64_t C, CKernel* k, CLabels* lab);
+
+		/** default destructor */
 		virtual ~CGMNPSVM();
 
-		/** train SVM */
-		virtual bool train();
+		/** train SVM
+		 *
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
+		 */
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
 		 * @return classifier type GMNPSVM
 		 */
 		virtual inline EClassifierType get_classifier_type() { return CT_GMNPSVM; }
-		
-		void getbasealphas(::std::vector< ::std::vector<float64_t> > & basealphas2);
+
+		/** required for CMKLMulticlass constraint computation
+		 *
+		 *  @param basealphas basealphas[k][j] is the alpha for class k and 
+		 * 	sample j which is untransformed compared to the alphas
+		 * 	stored in CSVM* members
+		 */
+		void getbasealphas(::std::vector< ::std::vector<float64_t> > & basealphas);
 
 		/** @return object name */
 		inline virtual const char* get_name() const { return "GMNPSVM"; }
 		
 	protected: 
-		::std::vector< ::std::vector<float64_t> > basealphas; // is the basic untransformed alpha, needed for MKL 
-		
+		/** required for CMKLMulticlass 
+		 * stores the untransformed alphas of this algorithm
+		 * whereas CSVM* members stores a transformed version of it
+		 * m_basealphas[k][j] is the alpha for class k and sample j
+		 */
+		::std::vector< ::std::vector<float64_t> > m_basealphas; // is the basic untransformed alpha, needed for MKL 
 };
+}
 #endif
