@@ -14,6 +14,8 @@
 #include "lib/common.h"
 #include "classifier/svm/SVM.h"
 
+namespace shogun
+{
 /** @brief class GNPPSVM */
 class CGNPPSVM : public CSVM
 {
@@ -31,8 +33,15 @@ class CGNPPSVM : public CSVM
 
 		virtual ~CGNPPSVM();
 
-		/** train SVM */
-		virtual bool train();
+		/** train SVM classifier
+		 *
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
+		 */
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
@@ -43,4 +52,5 @@ class CGNPPSVM : public CSVM
 		/** @return object name */
 		inline virtual const char* get_name() const { return "GNPPSVM"; }
 };
+}
 #endif
