@@ -16,6 +16,8 @@
 
 #include <stdio.h>
 
+namespace shogun
+{
 /** @brief class GPBTSVM */
 class CGPBTSVM : public CSVM
 {
@@ -32,11 +34,15 @@ class CGPBTSVM : public CSVM
 		CGPBTSVM(float64_t C, CKernel* k, CLabels* lab);
 		virtual ~CGPBTSVM();
 
-		/** train SVM
+		/** train SVM classifier
 		 *
-		 * @return if training was successful
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
 		 */
-		virtual bool train();
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
@@ -51,4 +57,5 @@ class CGPBTSVM : public CSVM
 		/** SVM model */
 		struct svm_model* model;
 };
+}
 #endif
