@@ -53,32 +53,32 @@ bool CLibSVM::train(CFeatures* data)
 
 
 	// check length of linear term
-	if (!linear_term.empty() && labels->get_num_labels() != (int32_t)linear_term.size())
+	if (!linear_term.empty() &&
+			labels->get_num_labels() != (int32_t)linear_term.size())
+	{
 		SG_ERROR("Number of training vectors does not match length of linear term\n");
+	}
 
 	// set linear term
-	if (!linear_term.empty()) {
-
+	if (!linear_term.empty())
+	{
 		// set with linear term from base class
 		problem.pv = get_linear_term_array();
 
-	} else {
-
+	}
+	else
+	{
 		// fill with minus ones
 		problem.pv = new float64_t[problem.l];
 
-		for (int i=0; i!=problem.l; i++) {
+		for (int i=0; i!=problem.l; i++)
 			problem.pv[i] = -1.0;
-		}
 	}
 
-
-
 	problem.y=new float64_t[problem.l];
 	problem.x=new struct svm_node*[problem.l];
     problem.C=new float64_t[problem.l];
 
-
 	x_space=new struct svm_node[2*problem.l];
 
 	for (int32_t i=0; i<problem.l; i++)
@@ -103,6 +103,7 @@ bool CLibSVM::train(CFeatures* data)
 	param.nu = get_nu();
 	param.kernel=kernel;
 	param.cache_size = kernel->get_cache_size();
+	param.max_train_time = max_train_time;
 	param.C = get_C1();
 	param.eps = epsilon;
 	param.p = 0.1;
