@@ -14,15 +14,16 @@
 #include "lib/common.h"
 #include "classifier/svm/MultiClassSVM.h"
 #include "classifier/svm/SVM_libsvm.h"
+#include "classifier/svm/LibSVM.h"
 
-#include <stdio.h>
-
+namespace shogun
+{
 /** @brief class LibSVMMultiClass */
 class CLibSVMMultiClass : public CMultiClassSVM
 {
 	public:
 		/** default constructor */
-		CLibSVMMultiClass();
+		CLibSVMMultiClass(LIBSVM_SOLVER_TYPE st=LIBSVM_C_SVC);
 
 		/** constructor
 		 *
@@ -33,11 +34,15 @@ class CLibSVMMultiClass : public CMultiClassSVM
 		CLibSVMMultiClass(float64_t C, CKernel* k, CLabels* lab);
 		virtual ~CLibSVMMultiClass();
 
-		/** train SVM
+		/** train multiclass SVM classifier
 		 *
-		 * @return if training was successful
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
 		 */
-		virtual bool train();
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
@@ -56,6 +61,9 @@ class CLibSVMMultiClass : public CMultiClassSVM
 
 		/** SVM model */
 		struct svm_model* model;
-};
-#endif
 
+		/** solver type */
+		LIBSVM_SOLVER_TYPE solver_type;
+};
+}
+#endif
