@@ -17,6 +17,8 @@
 
 #include <stdio.h>
 
+namespace shogun
+{
 /** @brief class LibSVMOneClass */
 class CLibSVMOneClass : public CSVM
 {
@@ -32,8 +34,15 @@ class CLibSVMOneClass : public CSVM
 		CLibSVMOneClass(float64_t C, CKernel* k);
 		virtual ~CLibSVMOneClass();
 
-		/** train SVM */
-		virtual bool train();
+		/** train SVM
+		 *
+		 * @param data training data (parameter can be avoided if distance or
+		 * kernel-based classifiers are used and distance/kernels are
+		 * initialized with train data)
+		 *
+		 * @return whether training was successful
+		 */
+		virtual bool train(CFeatures* data=NULL);
 
 		/** get classifier type
 		 *
@@ -53,4 +62,5 @@ class CLibSVMOneClass : public CSVM
 		/** SVM model */
 		struct svm_model* model;
 };
+}
 #endif
