@@ -15,6 +15,7 @@
 #include "features/Features.h"
 #include "kernel/Kernel.h"
 #include "classifier/KernelMachine.h"
+#include "lib/Parameter.h"
 
 namespace shogun
 {
@@ -98,15 +99,17 @@ class CSVM : public CKernelMachine
 		 */
 		inline void set_nu(float64_t nue) { nu=nue; }
 
+
 		/** set C
 		 *
-		 * @param c1 new C constant for negatively labelled examples
-		 * @param c2 new C constant for positively labelled examples
+		 * @param c_neg new C constant for negatively labeled examples
+		 * @param c_pos new C constant for positively labeled examples
 		 *
 		 * Note that not all SVMs support this (however at least CLibSVM and
 		 * CSVMLight do)
 		 */
-		inline void set_C(float64_t c1, float64_t c2) { C1=c1; C2=c2; }
+		inline void set_C(float64_t c_neg, float64_t c_pos) { C1=c_neg; C2=c_pos; }
+
 
 		/** set epsilon
 		 *
@@ -120,6 +123,12 @@ class CSVM : public CKernelMachine
 		 */
 		inline void set_tube_epsilon(float64_t eps) { tube_epsilon=eps; }
 
+		/** get tube epsilon
+		 *
+		 * @return tube epsilon
+		 */
+		inline float64_t get_tube_epsilon() { return tube_epsilon; }
+
 		/** set qpsize
 		 *
 		 * @param qps new qpsize
@@ -312,6 +321,8 @@ class CSVM : public CKernelMachine
 		/** mkl object that svm optimizers need to pass when calling the callback
 		 * function */
 		CMKL* mkl;
+
+		CParameter parameters;
 };
 }
 #endif
