@@ -1,3 +1,35 @@
+/*
+ * Copyright (c) 2007-2009 The LIBLINEAR Project.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * 
+ * 3. Neither name of copyright holders nor the names of its contributors
+ * may be used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ * 
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
 #ifndef _LIBLINEAR_H
@@ -67,31 +99,8 @@ struct model
 	float64_t bias;
 };
 
-struct model* train(const struct problem *prob, const struct parameter *param);
-void cross_validation(
-	const struct problem *prob, const struct parameter *param, int32_t nr_fold,
-	int32_t *target);
-
-int32_t predict_values(
-	const struct model *model_, const struct feature_node *x,
-	float64_t* dec_values);
-int32_t predict(const struct model *model_, const struct feature_node *x);
-int32_t predict_probability(
-	const struct model *model_, const struct feature_node *x,
-	float64_t* prob_estimates);
-
-int32_t save_model(const char *model_file_name, const struct model *model_);
-struct model *load_model(const char *model_file_name);
-
-int32_t get_nr_feature(const struct model *model_);
-int32_t get_nr_class(const struct model *model_);
-void get_labels(const struct model *model_, int32_t* label);
-
 void destroy_model(struct model *model_);
 void destroy_param(struct parameter *param);
-const char *check_parameter(
-	const struct problem *prob, const struct parameter *param);
-
 #ifdef __cplusplus
 }
 #endif
@@ -149,8 +158,8 @@ private:
 	const problem *prob;
 };
 
-/** class l2_lr_fun */
-class l2_lr_fun : public function
+/** class l2r_lr_fun */
+class l2r_lr_fun : public function
 {
 public:
 	/** constructor
@@ -159,8 +168,8 @@ public:
 	 * @param Cp Cp
 	 * @param Cn Cn
 	 */
-	l2_lr_fun(const problem *prob, float64_t Cp, float64_t Cn);
-	~l2_lr_fun();
+	l2r_lr_fun(const problem *prob, float64_t Cp, float64_t Cn);
+	~l2r_lr_fun();
 
 	/** fun
 	 *
@@ -194,6 +203,50 @@ private:
 	float64_t *D;
 	const problem *prob;
 };
+
+class l2r_l2_svc_fun : public function
+{
+public:
+	l2r_l2_svc_fun(const problem *prob, double Cp, double Cn);
+	~l2r_l2_svc_fun();
+
+	double fun(double *w);
+	void grad(double *w, double *g);
+	void Hv(double *s, double *Hs);
+
+	int get_nr_variable(void);
+
+private:
+	void Xv(double *v, double *Xv);
+	void subXv(double *v, double *Xv);
+	void subXTv(double *v, double *XTv);
+
+	double *C;
+	double *z;
+	double *D;
+	int *I;
+	int sizeI;
+	const problem *prob;
+};
+
+class Solver_MCSVM_CS
+{
+	public:
+		Solver_MCSVM_CS(const problem *prob, int nr_class, double *C, double eps=0.1, int max_iter=100000);
+		~Solver_MCSVM_CS();
+		void Solve(double *w);
+	private:
+		void solve_sub_problem(double A_i, int yi, double C_yi, int active_i, double *alpha_new);
+		bool be_shrunk(int i, int m, int yi, double alpha_i, double minG);
+		double *B, *C, *G;
+		int w_size, l;
+		int nr_class;
+		int max_iter;
+		double eps;
+		const problem *prob;
+};
+
+
 }
 #endif //HAVE_LAPACK
 #endif //_LIBLINEAR_H
