@@ -412,12 +412,12 @@ void CSubGradientSVM::init(int32_t num_vec, int32_t num_feat)
 	// alloc normal and bias inited with 0
 	delete[] w;
 	w=new float64_t[num_feat];
+	w_dim=num_feat;
 	memset(w,0,sizeof(float64_t)*num_feat);
 	//CMath::random_vector(w, num_feat, -1.0, 1.0);
 	bias=0;
 	num_it_noimprovement=0;
 	grad_b=0;
-	set_w(w, num_feat);
 	qpsize_limit=5000;
 
 	grad_w=new float64_t[num_feat];
