@@ -66,7 +66,7 @@
  *** Date:     November, 2006                                                ***
  *** Revision: 2                                                            ***
  ***                                                                        ***
- *** SHOGUN adaptions  Written (W) 2006-2008 Soeren Sonnenburg              ***
+ *** SHOGUN adaptions  Written (W) 2006-2009 Soeren Sonnenburg              ***
  ******************************************************************************/
 #include <math.h>
 #include <stdio.h>
@@ -80,7 +80,11 @@
 #include "lib/Signal.h"
 #include "lib/io.h"
 
+using namespace shogun;
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
+namespace shogun
+{
 #define y_in(i)      y[index_in[(i)]]
 #define y_out(i)     y[index_out[(i)]]
 #define alpha_in(i)  alpha[index_in[(i)]]
@@ -1564,6 +1568,7 @@ void quick_s3(int32_t a[], int32_t n, int32_t ia[])
       } while (s < d);
   } while (l >= 0);
 }
+}
 
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
