@@ -14,6 +14,13 @@
 #include "features/Alphabet.h"
 #include "lib/io.h"
 
+
+#ifdef HAVE_BOOST_SERIALIZATION
+#include <boost/serialization/export.hpp>
+BOOST_CLASS_EXPORT(shogun::CAlphabet);
+#endif //HAVE_BOOST_SERIALIZATION
+
+
 using namespace shogun;
 
 //define numbers for the bases
@@ -21,8 +28,13 @@ const uint8_t CAlphabet::B_A=0;
 const uint8_t CAlphabet::B_C=1;
 const uint8_t CAlphabet::B_G=2;
 const uint8_t CAlphabet::B_T=3;
+const uint8_t CAlphabet::B_0=4;
 const uint8_t CAlphabet::MAPTABLE_UNDEF=0xff;
-const char* CAlphabet::alphabet_names[12]={"DNA", "RAWDNA", "RNA", "PROTEIN", "BINARY", "ALPHANUM", "CUBE", "RAW", "IUPAC_NUCLEIC_ACID", "IUPAC_AMINO_ACID", "NONE", "UNKNOWN"};
+const char* CAlphabet::alphabet_names[18]={
+	"DNA","RAWDNA", "RNA", "PROTEIN", "BINARY", "ALPHANUM",
+	"CUBE", "RAW", "IUPAC_NUCLEIC_ACID", "IUPAC_AMINO_ACID",
+	"NONE", "DIGIT", "DIGIT2", "RAWDIGIT", "RAWDIGIT2", "UNKNOWN",
+	"SNP", "RAWSNP"};
 
 /*
 CAlphabet::CAlphabet()
@@ -51,6 +63,18 @@ CAlphabet::CAlphabet(char* al, int32_t len)
 		alpha = ALPHANUM;
 	else if (len>=(int32_t) strlen("CUBE") && !strncmp(al, "CUBE", strlen("CUBE")))
 		alpha = CUBE;
+	else if (len>=(int32_t) strlen("DIGIT2") && !strncmp(al, "DIGIT2", strlen("DIGIT2")))
+		alpha = DIGIT2;
+	else if (len>=(int32_t) strlen("DIGIT") && !strncmp(al, "DIGIT", strlen("DIGIT")))
+		alpha = DIGIT;
+	else if (len>=(int32_t) strlen("RAWDIGIT2") && !strncmp(al, "RAWDIGIT2", strlen("RAWDIGIT2")))
+		alpha = RAWDIGIT2;
+	else if (len>=(int32_t) strlen("RAWDIGIT") && !strncmp(al, "RAWDIGIT", strlen("RAWDIGIT")))
+		alpha = RAWDIGIT;
+	else if (len>=(int32_t) strlen("SNP") && !strncmp(al, "SNP", strlen("SNP")))
+		alpha = SNP;
+	else if (len>=(int32_t) strlen("RAWSNP") && !strncmp(al, "RAWSNP", strlen("RAWSNP")))
+		alpha = RAWSNP;
 	else if ((len>=(int32_t) strlen("BYTE") && !strncmp(al, "BYTE", strlen("BYTE"))) ||
 			(len>=(int32_t) strlen("RAW") && !strncmp(al, "RAW", strlen("RAW"))))
 		alpha = RAWBYTE;
@@ -121,6 +145,24 @@ bool CAlphabet::set_alphabet(EAlphabet alpha)
 		case NONE:
 			num_symbols = 0;
 			break;
+		case DIGIT2:
+			num_symbols = 3;
+			break;
+		case DIGIT:
+			num_symbols = 10;
+			break;
+		case RAWDIGIT2:
+			num_symbols = 3;
+			break;
+		case RAWDIGIT:
+			num_symbols = 10;
+			break;
+		case SNP:
+			num_symbols = 5;
+			break;
+		case RAWSNP:
+			num_symbols = 5;
+			break;
 		default:
 			num_symbols = 0;
 			result=false;
@@ -138,8 +180,7 @@ bool CAlphabet::set_alphabet(EAlphabet alpha)
 
 void CAlphabet::init_map_table()
 {
-	int32_t i;
-	for (i=0; i<(1<<(8*sizeof(uint8_t))); i++)
+	for (int32_t i=0; i<(1<<(8*sizeof(uint8_t))); i++)
 	{
 		maptable_to_bin[i] = MAPTABLE_UNDEF;
 		maptable_to_char[i] = MAPTABLE_UNDEF;
@@ -148,6 +189,73 @@ void CAlphabet::init_map_table()
 
 	switch (alphabet)
 	{
+		case RAWDIGIT:
+			for (uint8_t i=0; i<=9; i++)
+			{
+				valid_chars[i]=true;
+				maptable_to_bin[i]=i;
+				maptable_to_char[i]=i;
+			}
+			break;
+
+		case RAWDIGIT2:
+			for (uint8_t i=0; i<=2; i++)
+			{
+				valid_chars[i]=true;
+				maptable_to_bin[i]=i;
+				maptable_to_char[i]=i;
+			}
+			break;
+
+		case DIGIT:
+			valid_chars[(uint8_t) '0']=true;
+			valid_chars[(uint8_t) '1']=true;
+			valid_chars[(uint8_t) '2']=true;
+			valid_chars[(uint8_t) '3']=true;
+			valid_chars[(uint8_t) '4']=true;
+			valid_chars[(uint8_t) '5']=true;
+			valid_chars[(uint8_t) '6']=true;
+			valid_chars[(uint8_t) '7']=true;
+			valid_chars[(uint8_t) '8']=true;
+			valid_chars[(uint8_t) '9']=true;	//Translation '0-9' -> 0-9
+
+			maptable_to_bin[(uint8_t) '0']=0;
+			maptable_to_bin[(uint8_t) '1']=1;
+			maptable_to_bin[(uint8_t) '2']=2;
+			maptable_to_bin[(uint8_t) '3']=3;
+			maptable_to_bin[(uint8_t) '4']=4;
+			maptable_to_bin[(uint8_t) '5']=5;
+			maptable_to_bin[(uint8_t) '6']=6;
+			maptable_to_bin[(uint8_t) '7']=7;
+			maptable_to_bin[(uint8_t) '8']=8;
+			maptable_to_bin[(uint8_t) '9']=9;	//Translation '0-9' -> 0-9
+
+			maptable_to_char[(uint8_t) 0]='0';
+			maptable_to_char[(uint8_t) 1]='1';
+			maptable_to_char[(uint8_t) 2]='2';
+			maptable_to_char[(uint8_t) 3]='3';
+			maptable_to_char[(uint8_t) 4]='4';
+			maptable_to_char[(uint8_t) 5]='5';
+			maptable_to_char[(uint8_t) 6]='6';
+			maptable_to_char[(uint8_t) 7]='7';
+			maptable_to_char[(uint8_t) 8]='8';
+			maptable_to_char[(uint8_t) 9]='9';	//Translation 0-9 -> '0-9'
+			break;
+
+		case DIGIT2:
+			valid_chars[(uint8_t) '0']=true;
+			valid_chars[(uint8_t) '1']=true;
+			valid_chars[(uint8_t) '2']=true; //Translation '0-2' -> 0-2
+
+			maptable_to_bin[(uint8_t) '0']=0;
+			maptable_to_bin[(uint8_t) '1']=1;
+			maptable_to_bin[(uint8_t) '2']=2;	//Translation '0-2' -> 0-2
+
+			maptable_to_char[(uint8_t) 0]='0';
+			maptable_to_char[(uint8_t) 1]='1';
+			maptable_to_char[(uint8_t) 2]='2'; //Translation 0-2 -> '0-2'
+			break;
+
 		case CUBE:
 			valid_chars[(uint8_t) '1']=true;
 			valid_chars[(uint8_t) '2']=true;
@@ -174,7 +282,7 @@ void CAlphabet::init_map_table()
 		case PROTEIN:
 			{
 				int32_t skip=0 ;
-				for (i=0; i<21; i++)
+				for (int32_t i=0; i<21; i++)
 				{
 					if (i==1) skip++ ;
 					if (i==8) skip++ ;
@@ -200,13 +308,13 @@ void CAlphabet::init_map_table()
 
 		case ALPHANUM:
 			{
-				for (i=0; i<26; i++)
+				for (int32_t i=0; i<26; i++)
 				{
 					valid_chars['A'+i]=true;
 					maptable_to_bin['A'+i]=i ;
 					maptable_to_char[i]='A'+i ;
 				} ;
-				for (i=0; i<10; i++)
+				for (int32_t i=0; i<10; i++)
 				{
 					valid_chars['0'+i]=true;
 					maptable_to_bin['0'+i]=26+i ;
@@ -218,7 +326,7 @@ void CAlphabet::init_map_table()
 		case RAWBYTE:
 			{
 				//identity
-				for (i=0; i<256; i++)
+				for (int32_t i=0; i<256; i++)
 				{
 					valid_chars[i]=true;
 					maptable_to_bin[i]=i;
@@ -246,7 +354,38 @@ void CAlphabet::init_map_table()
 		case RAWDNA:
 			{
 				//identity
-				for (i=0; i<4; i++)
+				for (int32_t i=0; i<4; i++)
+				{
+					valid_chars[i]=true;
+					maptable_to_bin[i]=i;
+					maptable_to_char[i]=i;
+				}
+			}
+			break;
+
+		case SNP:
+			valid_chars[(uint8_t) 'A']=true;
+			valid_chars[(uint8_t) 'C']=true;
+			valid_chars[(uint8_t) 'G']=true;
+			valid_chars[(uint8_t) 'T']=true;
+			valid_chars[(uint8_t) '0']=true;
+
+			maptable_to_bin[(uint8_t) 'A']=B_A;
+			maptable_to_bin[(uint8_t) 'C']=B_C;
+			maptable_to_bin[(uint8_t) 'G']=B_G;
+			maptable_to_bin[(uint8_t) 'T']=B_T;
+			maptable_to_bin[(uint8_t) '0']=B_0;
+
+			maptable_to_char[B_A]='A';
+			maptable_to_char[B_C]='C';
+			maptable_to_char[B_G]='G';
+			maptable_to_char[B_T]='T';
+			maptable_to_char[B_0]='0';
+			break;
+		case RAWSNP:
+			{
+				//identity
+				for (int32_t i=0; i<5; i++)
 				{
 					valid_chars[i]=true;
 					maptable_to_bin[i]=i;
@@ -538,46 +677,16 @@ const char* CAlphabet::get_alphabet_name(EAlphabet alphabet)
 		case NONE:
 			idx=10;
 			break;
-		default:
+		case DIGIT:
 			idx=11;
 			break;
+		case DIGIT2:
+			idx=12;
+			break;
+		default:
+			idx=13;
+			break;
 	}
 	return alphabet_names[idx];
 }
 
-#ifdef HAVE_BOOST_SERIALIZATION
-std::string CAlphabet::to_string() const
-{
-	std::ostringstream s;
-	::boost::archive::text_oarchive oa(s);
-	oa << this;
-	return s.str();
-}
-
-void CAlphabet::from_string(std::string str)
-{
-	std::istringstream is(str);
-	::boost::archive::text_iarchive ia(is);
-
-	//cast away constness
-	CAlphabet* tmp = const_cast<CAlphabet*>(this);
-
-	ia >> tmp;
-	*this = *tmp;
-}
-
-void CAlphabet::to_file(std::string filename) const
-{
-	std::ofstream os(filename.c_str(), std::ios::binary);
-	::boost::archive::binary_oarchive oa(os);
-	oa << this;
-}
-
-void CAlphabet::from_file(std::string filename)
-{
-	std::ifstream is(filename.c_str(), std::ios::binary);
-	::boost::archive::binary_iarchive ia(is);
-	CAlphabet* tmp= const_cast<CAlphabet*>(this);
-	ia >> tmp;
-}
-#endif //HAVE_BOOST_SERIALIZATION
