@@ -4,8 +4,9 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2009 Soeren Sonnenburg
+ * Written (W) 2009-2010 Soeren Sonnenburg
  * Copyright (C) 2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #include "features/CombinedDotFeatures.h"
@@ -116,6 +117,71 @@ float64_t CCombinedDotFeatures::dense_dot(int32_t vec_idx1, const float64_t* vec
 	return result;
 }
 
+void CCombinedDotFeatures::dense_dot_range(float64_t* output, int32_t start, int32_t stop, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b)
+{
+	if (stop<=start)
+		return;
+	ASSERT(dim==num_dimensions);
+
+	CListElement<CDotFeatures*> * current = NULL;
+	CDotFeatures* f=get_first_feature_obj(current);
+	uint32_t offs=0;
+	bool first=true;
+	int32_t num=stop-start;
+	float64_t* tmp=new float64_t[num];
+
+	while (f)
+	{
+		int32_t f_dim = f->get_dim_feature_space();
+		if (first)
+		{
+			f->dense_dot_range(output, start, stop, alphas, vec+offs, f_dim, b);
+			first=false;
+		}
+		else
+		{
+			f->dense_dot_range(tmp, start, stop, alphas, vec+offs, f_dim, b);
+			for (int32_t i=0; i<num; i++)
+				output[i]+=tmp[i];
+		}
+		offs += f_dim;
+		f=get_next_feature_obj(current);
+	}
+	delete[] tmp;
+}
+
+void CCombinedDotFeatures::dense_dot_range_subset(int32_t* sub_index, int32_t num, float64_t* output, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b)
+{
+	if (num<=0)
+		return;
+	ASSERT(dim==num_dimensions);
+
+	CListElement<CDotFeatures*> * current = NULL;
+	CDotFeatures* f=get_first_feature_obj(current);
+	uint32_t offs=0;
+	bool first=true;
+	float64_t* tmp=new float64_t[num];
+
+	while (f)
+	{
+		int32_t f_dim = f->get_dim_feature_space();
+		if (first)
+		{
+			f->dense_dot_range_subset(sub_index, num, output, alphas, vec+offs, f_dim, b);
+			first=false;
+		}
+		else
+		{
+			f->dense_dot_range_subset(sub_index, num, tmp, alphas, vec+offs, f_dim, b);
+			for (int32_t i=0; i<num; i++)
+				output[i]+=tmp[i];
+		}
+		offs += f_dim;
+		f=get_next_feature_obj(current);
+	}
+	delete[] tmp;
+}
+
 void CCombinedDotFeatures::add_to_dense_vec(float64_t alpha, int32_t vec_idx1, float64_t* vec2, int32_t vec2_len, bool abs_val)
 {
 	CListElement<CDotFeatures*> * current = NULL ;
