@@ -4,8 +4,10 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2009 Soeren Sonnenburg
+ * Written (W) 2009-2010 Soeren Sonnenburg
  * Copyright (C) 2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
+ *
  */
 
 #ifndef _COMBINEDDOTFEATURES_H___
@@ -52,6 +54,10 @@ class CCombinedDotFeatures : public CDotFeatures
 		/** destructor */
 		virtual ~CCombinedDotFeatures();
 
+		/** get the number of vectors
+		 *
+		 * @return number of vectors
+		 */
 		inline virtual int32_t get_num_vectors()
 		{
 			return num_vectors;
@@ -82,6 +88,32 @@ class CCombinedDotFeatures : public CDotFeatures
 		 */
 		virtual float64_t dense_dot(int32_t vec_idx1, const float64_t* vec2, int32_t vec2_len);
 
+		/** Compute the dot product for a range of vectors. This function makes use of dense_dot
+		 * alphas[i] * sparse[i]^T * w + b
+		 *
+		 * @param output result for the given vector range
+		 * @param start start vector range from this idx
+		 * @param stop stop vector range at this idx
+		 * @param alphas scalars to multiply with, may be NULL
+		 * @param vec dense vector to compute dot product with
+		 * @param dim length of the dense vector
+		 * @param b bias
+		 */
+		virtual void dense_dot_range(float64_t* output, int32_t start, int32_t stop, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b);
+
+		/** Compute the dot product for a subset of vectors. This function makes use of dense_dot
+		 * alphas[i] * sparse[i]^T * w + b
+		 *
+		 * @param sub_index index for which to compute outputs
+		 * @param num length of index
+		 * @param output result for the given vector range
+		 * @param alphas scalars to multiply with, may be NULL
+		 * @param vec dense vector to compute dot product with
+		 * @param dim length of the dense vector
+		 * @param b bias
+		 */
+		virtual void dense_dot_range_subset(int32_t* sub_index, int32_t num, float64_t* output, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b);
+
 		/** add vector 1 multiplied with alpha to dense vector2
 		 *
 		 * @param alpha scalar alpha
@@ -117,11 +149,97 @@ class CCombinedDotFeatures : public CDotFeatures
 			return C_COMBINED_DOT;
 		}
 
+		/** get the size of a single element
+		 *
+		 * @return size of a element
+		 */
 		inline virtual int32_t get_size()
 		{
 			return sizeof(float64_t);
 		}
 
+		/** iterator for combined dotfeatures */
+		struct combined_feature_iterator
+		{
+			/** pointer to current feature object */
+			CDotFeatures* f;
+			/** pointer to list object */
+			CListElement<CDotFeatures*>* current;
+			/** pointer to combined feature iterator */
+			void* iterator;
+			/* the index of the vector over whose components to iterate over */
+			int32_t vector_index;
+		};
+
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index)
+		{
+			combined_feature_iterator* it=new combined_feature_iterator[1];
+
+			it->current=NULL;
+			it->f=get_first_feature_obj(it->current);
+			it->iterator=it->f->get_feature_iterator(vector_index);
+			it->vector_index=vector_index;
+			return it;
+		}
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator)
+		{
+			ASSERT(iterator);
+			combined_feature_iterator* it = (combined_feature_iterator*) iterator;
+
+			while (it->f)
+			{
+				if (it->f->get_next_feature(index, value, it->iterator))
+				{
+					value*=get_combined_feature_weight();
+					return true;
+				}
+
+				it->f->free_feature_iterator(it->iterator);
+				it->f=get_next_feature_obj(it->current);
+				if (it->f)
+					it->iterator=it->f->get_feature_iterator(it->vector_index);
+				else
+					it->iterator=NULL;
+			}
+			return false;
+		}
+
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator)
+		{
+			if (iterator)
+			{
+				combined_feature_iterator* it = (combined_feature_iterator*) iterator;
+				if (it->iterator && it->f)
+					it->f->free_feature_iterator(it->iterator);
+				delete[] it;
+			}
+		}
+
 		/** duplicate feature object
 		 *
 		 * @return feature object
