@@ -4,14 +4,17 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2009 Soeren Sonnenburg
+ * Written (W) 2009-2010 Soeren Sonnenburg
  * Copyright (C) 2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #ifndef _DOTFEATURES_H___
 #define _DOTFEATURES_H___
 
 #include "lib/common.h"
+#include "lib/Time.h"
+#include "lib/Mathematics.h"
 #include "features/Features.h"
 
 namespace shogun
@@ -30,9 +33,11 @@ namespace shogun
  *
  *   \f[r = {\bf x} \cdot {\bf z}\f]
  *
- * - multiplication with a scalar \f$\alpha\f$ and addition on to a dense vector \f${\bf z}\f$:
+ * - multiplication with a scalar \f$\alpha\f$ and addition to a dense vector \f${\bf z}\f$:
  *
- *   \f[{\bf z'} = \alpha {\bf x} + {\bf z}\f]
+ *   \f[ {\bf z'} = \alpha {\bf x} + {\bf z} \f]
+ *
+ * - iteration over all (potentially) non-zero features of \f${\bf x}\f$
  * 
  */
 class CDotFeatures : public CFeatures
@@ -54,9 +59,9 @@ class CDotFeatures : public CFeatures
 
 		/** constructor
 		 *
-		 * @param fname filename to load features from
+		 * @param loader File object via which to load data
 		 */
-		CDotFeatures(char* fname) : CFeatures(fname) {}
+		CDotFeatures(CFile* loader) : CFeatures(loader) {}
 
 		virtual ~CDotFeatures() { }
 
@@ -105,9 +110,23 @@ class CDotFeatures : public CFeatures
 		 * @param vec dense vector to compute dot product with
 		 * @param dim length of the dense vector
 		 * @param b bias
+		 *
+		 * note that the result will be written to output[0...(stop-start-1)]
 		 */
 		virtual void dense_dot_range(float64_t* output, int32_t start, int32_t stop, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b);
 
+		/** Compute the dot product for a subset of vectors. This function makes use of dense_dot
+		 * alphas[i] * sparse[i]^T * w + b
+		 *
+		 * @param sub_index index for which to compute outputs
+		 * @param num length of index
+		 * @param output result for the given vector range
+		 * @param alphas scalars to multiply with, may be NULL
+		 * @param vec dense vector to compute dot product with
+		 * @param dim length of the dense vector
+		 * @param b bias
+		 */
+		virtual void dense_dot_range_subset(int32_t* sub_index, int32_t num, float64_t* output, float64_t* alphas, float64_t* vec, int32_t dim, float64_t b);
 
 		/** Compute the dot product for a range of vectors. This function is
 		 * called by the threads created in dense_dot_range */
@@ -143,6 +162,51 @@ class CDotFeatures : public CFeatures
 		 */
 		virtual void get_feature_matrix(float64_t** dst, int32_t* num_feat, int32_t* num_vec);
 
+		/** get a copy of the feature matrix (in feature space)
+		 * num_feat,num_vectors are returned by reference
+		 *
+		 * @param dst destination to store matrix in
+		 * @param len number of features
+		 * @param num index of vector
+		 */
+		virtual void get_feature_vector(float64_t** dst, int32_t* len, int32_t num);
+
+		/** run benchmark for add_to_dense_vec */
+		void benchmark_add_to_dense_vector(int32_t repeats=5);
+
+		/** run benchmark for dense_dot_range */
+		void benchmark_dense_dot_range(int32_t repeats=5);
+
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index)=0;
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator)=0;
+
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator)=0;
+
 	protected:
 		/** display progress output
 		 *
