@@ -4,8 +4,9 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2009 Soeren Sonnenburg
+ * Written (W) 2009-2010 Soeren Sonnenburg
  * Copyright (C) 2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #ifndef _IMPLICITSPECFEATURES_H___
@@ -93,8 +94,14 @@ class CImplicitWeightedSpecFeatures : public CDotFeatures
 		 */
 		virtual inline int32_t get_nnz_features_for_vector(int32_t num)
 		{
-			SG_NOTIMPLEMENTED;
-			return 0;
+			int32_t vlen=-1;
+			bool free_vec;
+			uint16_t* vec1=strings->get_feature_vector(num, vlen, free_vec);
+			strings->free_feature_vector(vec1, num, free_vec);
+			int32_t nnz=0;
+			for (int32_t i=1; i<=degree; i++)
+				nnz+=CMath::min(CMath::pow(alphabet_size,i), vlen);
+			return nnz;
 		}
 
 		/** get feature type
@@ -147,6 +154,60 @@ class CImplicitWeightedSpecFeatures : public CDotFeatures
 		 */
 		bool set_weights(float64_t* w, int32_t d);
 
+		/** iterator for weighted spectrum features */
+		struct wspec_feature_iterator
+		{
+			/** pointer to feature vector */
+			uint16_t* vec;
+			/** index of vector */
+			int32_t vidx;
+			/** length of vector */
+			int32_t vlen;
+			/** if we need to free the vector*/
+			bool vfree;
+
+			/** @name Internal Parameters
+			 * parameters of interal feature gen loop
+			 */
+			//@{
+			int32_t offs;
+			int32_t d;
+			int32_t j;
+			uint8_t mask;
+			float64_t alpha;
+			//@}
+		};
+
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index);
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator);
+
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator);
+
 		/** @return object name */
 		inline virtual const char* get_name() const { return "ImplicitWeightedSpecFeatures"; }
 
