@@ -14,10 +14,14 @@
 
 #include "lib/common.h"
 #include "lib/io.h"
+#include "lib/File.h"
 #include "base/SGObject.h"
 
 namespace shogun
 {
+
+	class CFile;
+
 /** @brief The class Labels models labels, i.e. class assignments of objects.
  *
  * Labels here are always real-valued and thus applicable to classification
@@ -42,26 +46,31 @@ class CLabels : public CSGObject
 		 */
 		CLabels(float64_t* src, int32_t len);
 
+		/** constructor 
+		 * @param in_confidences confidence matrix to be used to derive the labels
+		 * @param in_num_labels number of labels
+		 * @param in_num_classes number of classes
+		 */
+		CLabels(float64_t* in_confidences, int32_t in_num_labels, int32_t in_num_classes);
+
 		/** constructor
 		 *
-		 * @param fname filename to load labels from
+		 * @param loader File object via which to load data
 		 */
-		CLabels(char* fname);
+		CLabels(CFile* loader);
 		virtual ~CLabels();
 
 		/** load labels from file
 		 *
-		 * @param fname filename to load from
-		 * @return if loading was successful
+		 * @param loader File object via which to load data
 		 */
-		bool load(char* fname);
+		virtual void load(CFile* loader);
 
 		/** save labels to file
 		 *
-		 * @param fname filename to save to
-		 * @return if saving was successful
+		 * @param writer File object via which to save data
 		 */
-		bool save(char* fname);
+		virtual void save(CFile* writer);
 
 		/** set label
 		 *
@@ -162,6 +171,34 @@ class CLabels : public CSGObject
 		 */
 		void set_labels(float64_t* src, int32_t len);
 
+		/** set confidences 
+		 * @param in_confidences confidence matrix to be used to derive the labels
+		 * @param in_num_labels number of labels
+		 * @param in_num_classes number of classes
+		 */
+		void set_confidences(float64_t* in_confidences, int32_t in_num_labels, int32_t in_num_classes);
+
+		/** get confidences 
+		 * @param out_num_labels number of labels
+		 * @param out_num_classes number of classes will be written to it
+		 * @return pointer to the confidences matrix
+		 */
+		float64_t* get_confidences(int32_t& out_num_labels, int32_t& out_num_classes);
+
+		/** get confidences  (swig compatible)
+		 * @param dst pointer to the confidences matrix (returned)
+		 * @param out_num_labels number of labels (returned)
+		 * @param out_num_classes number of classes will be written to it (returned)
+		 */
+		void get_confidences(float64_t** dst, int32_t* out_num_labels, int32_t* out_num_classes);
+
+		/** get confidences for a sample
+		 * @param in_sample_index index of a sample
+		 * @param out_num_classes number of classes will be written to it
+		 * @return pointer to the confidences vector
+		 */
+		float64_t* get_sample_confidences(const int32_t& in_sample_index, int32_t& out_num_classes);
+
 		/** get INT label vector
 		 * caller has to clean up
 		 *
@@ -187,6 +224,10 @@ class CLabels : public CSGObject
 		/** @return object name */
 		inline virtual const char* get_name() const { return "Labels"; }
 
+	protected:
+		/** find labels from the confidences using argmax over the classes. */
+		void find_labels();
+
 #ifdef HAVE_BOOST_SERIALIZATION
     private:
 
@@ -273,6 +314,13 @@ class CLabels : public CSGObject
 		int32_t num_labels;
 		/** the labels */
 		float64_t* labels;
+
+		/** number of classes */
+		int16_t m_num_classes;
+
+		/** confidence matrix of size: num_classes x num_labels */
+		float64_t* m_confidences;
+
 };
 }
 #endif
