@@ -4,9 +4,10 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2009 Soeren Sonnenburg
+ * Written (W) 1999-2010 Soeren Sonnenburg
  * Written (W) 1999-2008 Gunnar Raetsch
  * Copyright (C) 1999-2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #ifndef _SIMPLEFEATURES__H__
@@ -28,6 +29,7 @@ namespace shogun
 template <class ST> class CStringFeatures;
 template <class ST> class CSimpleFeatures;
 template <class ST> class CSimplePreProc;
+class CDotFeatures;
 
 /** @brief The class SimpleFeatures implements dense feature matrices.
  *
@@ -80,7 +82,7 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			{
 				free_feature_matrix();
 				feature_matrix=new ST(num_vectors*num_features);
-				memcpy(feature_matrix, orig.feature_matrix, sizeof(float64_t)*num_vectors*num_features);
+				memcpy(feature_matrix, orig.feature_matrix, sizeof(float64_t)*int64_t(num_vectors)*num_features);
 			}
 		}
 
@@ -97,15 +99,16 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			copy_feature_matrix(src, num_feat, num_vec);
 		}
 
-		/** constructor
+		/** constructor loading features from file
 		 *
-		 * NOT IMPLEMENTED!
-		 *
-		 * @param fname filename to load features from
+		 * @param loader File object via which to load data
 		 */
-		CSimpleFeatures(char* fname)
-		: CDotFeatures(fname), num_vectors(0), num_features(0),
-			feature_matrix(NULL), feature_cache(NULL) {}
+		CSimpleFeatures(CFile* loader)
+		: CDotFeatures(loader), num_vectors(0), num_features(0),
+			feature_matrix(NULL), feature_cache(NULL)
+		{
+			load(loader);
+		}
 
 		/** duplicate feature object
 		 *
@@ -160,7 +163,7 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			if (feature_matrix)
 			{
 				dofree=false;
-				return &feature_matrix[num*num_features];
+				return &feature_matrix[num*int64_t(num_features)];
 			} 
 			else
 			{
@@ -235,7 +238,7 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			if (len != num_features)
 				SG_ERROR("Vector not of length %d (has %d)\n", num_features, len);
 
-			memcpy(&feature_matrix[num*num_features], src, num_features*sizeof(ST));
+			memcpy(&feature_matrix[num*int64_t(num_features)], src, int64_t(num_features)*sizeof(ST));
 		}
 
 		/** get feature vector num
@@ -290,10 +293,12 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 		{
 			ASSERT(feature_matrix);
 
-			int64_t num=num_features*num_vectors;
+			int64_t num=int64_t(num_features)*num_vectors;
 			*num_feat=num_features;
 			*num_vec=num_vectors;
 			*dst=(ST*) malloc(sizeof(ST)*num);
+			if (!*dst)
+				SG_ERROR("Allocating %ld bytes failes\n", sizeof(ST)*num);
 			memcpy(*dst, feature_matrix, num * sizeof(ST));
 		}
 
@@ -311,6 +316,50 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			return feature_matrix;
 		}
 
+		/** get a transposed copy of the features
+		 *
+		 * @return transposed copy
+		 */
+		CSimpleFeatures<ST>* get_transposed()
+		{
+			int32_t num_feat;
+			int32_t num_vec;
+			ST* fm=get_transposed(num_feat, num_vec);
+
+			return new CSimpleFeatures<ST>(fm, num_feat, num_vec);
+		}
+
+		/** compute and return the transpose of the feature matrix
+		 * which will be prepocessed.
+		 * num_feat, num_vectors are returned by reference
+		 * caller has to clean up
+		 *
+		 * @param num_feat number of features in matrix
+		 * @param num_vec number of vectors in matrix
+		 * @return transposed sparse feature matrix
+		 */
+		ST* get_transposed(int32_t &num_feat, int32_t &num_vec)
+		{
+			num_feat=num_vectors;
+			num_vec=num_features;
+
+			ST* fm=new ST[int64_t(num_feat)*num_vec];
+
+			for (int32_t i=0; i<num_vectors; i++)
+			{
+				int32_t vlen;
+				bool vfree;
+				ST* vec=get_feature_vector(i, vlen, vfree);
+
+				for (int32_t j=0; j<vlen; j++)
+					fm[j*int64_t(num_vectors)+i]=vec[j];
+
+				free_feature_vector(vec, i, vfree);
+			}
+
+			return fm;
+		}
+
 		/** set feature matrix
 		 * necessary to set feature_matrix, num_features,
 		 * num_vectors, where num_features is the column offset,
@@ -348,6 +397,36 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 			num_vectors=num_vec;
 		}
 
+		/** obtain simple features from other dotfeatures
+		 *
+		 * @param df dotfeatures to obtain features from
+		 */
+		void obtain_from_dot(CDotFeatures* df)
+		{
+			int32_t num_feat=df->get_dim_feature_space();
+			int32_t num_vec=df->get_num_vectors();
+
+			ASSERT(num_feat>0 && num_vec>0);
+
+			free_feature_matrix();
+			feature_matrix=new ST[((int64_t) num_feat)*num_vec];
+
+			for (int32_t i=0; i<num_vec; i++)
+			{
+				float64_t* dst;
+				int32_t len;
+				df->get_feature_vector(&dst, &len, i);
+				ASSERT(num_feat==len);
+
+				for (int32_t j=0; j<num_feat; j++)
+					feature_matrix[i*int64_t(num_feat)+j]=(ST) dst[j];
+
+				delete[] dst;
+			}
+			num_features=num_feat;
+			num_vectors=num_vec;
+		}
+
 		/** apply preprocessor
 		 *
 		 * @param force_preprocessing if preprocssing shall be forced
@@ -566,59 +645,93 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 
 		/** load features from file
 		 *
-		 * @param fname filename to load from
-		 * @return if loading was successful
+		 * @param loader File object via which to load data
 		 */
-		virtual bool load(char* fname)
-		{
-			bool status=false;
-			num_vectors=1;
-			num_features=0;
-			CFile f(fname, 'r', get_feature_type());
-			int64_t numf=0;
-			free_feature_matrix();
-			feature_matrix=f.load_data<ST>(NULL, numf);
-			num_features=numf;
-
-			if (!f.is_ok())
-				SG_ERROR( "loading file \"%s\" failed", fname);
-			else
-				status=true;
-
-			return status;
-		}
+		virtual inline void load(CFile* loader);
 
 		/** save features to file
 		 *
-		 * @param fname filename to save to
-		 * @return if saving was successful
+		 * @param saver File object via which to save data
 		 */
-		virtual bool save(char* fname)
+		virtual inline void save(CFile* saver);
+
+		/** iterator for simple features */
+		struct simple_feature_iterator
 		{
-			int32_t len;
-			bool free;
-			ST* fv;
+			/** pointer to feature vector */
+			ST* vec;
+			/** index of vector */
+			int32_t vidx;
+			/** length of vector */
+			int32_t vlen;
+			/** if we need to free the vector*/
+			bool vfree;
 
-			CFile f(fname, 'w', get_feature_type());
+			/** feature index */
+			int32_t index;
+		};
 
-			for (int32_t i=0; i< (int32_t) num_vectors && f.is_ok(); i++)
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index)
+		{
+			if (vector_index>=num_vectors)
 			{
-				if (!(i % (num_vectors/10+1)))
-					SG_PRINT( "%02d%%.", (int) (100.0*i/num_vectors));
-				else if (!(i % (num_vectors/200+1)))
-					SG_PRINT( ".");
-
-				fv=get_feature_vector(i, len, free);
-				f.save_data<ST>(fv, len);
-				free_feature_vector(fv, i, free) ;
+				SG_ERROR("Index out of bounds (number of vectors %d, you "
+						"requested %d)\n", num_vectors, vector_index);
 			}
 
-			if (f.is_ok())
-				SG_INFO( "%d vectors with %d features each successfully written (filesize: %ld)\n", num_vectors, num_features, num_vectors*num_features*sizeof(float64_t));
+			simple_feature_iterator* iterator=new simple_feature_iterator[1];
+			iterator->vec= get_feature_vector(vector_index, iterator->vlen, iterator->vfree);
+			iterator->vidx=vector_index;
+			iterator->index=0;
+			return iterator;
+		}
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator)
+		{
+			simple_feature_iterator* it=(simple_feature_iterator*) iterator;
+			if (!it || it->index>=it->vlen)
+				return false;
+
+			index=it->index++;
+			value = (float64_t) it->vec[index];
 
 			return true;
 		}
 
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator)
+		{
+			if (!iterator)
+				return;
+
+			simple_feature_iterator* it=(simple_feature_iterator*) iterator;
+			free_feature_vector(it->vec, it->vidx, it->vfree);
+			delete[] it;
+		}
+
 		/** @return object name */
 		inline virtual const char* get_name() const { return "SimpleFeatures"; }
 
@@ -668,7 +781,7 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
                 ar & num_vectors;
                 ar & num_features;
 
-                feature_matrix = new ST[num_vectors*num_features];
+                feature_matrix = new ST[int64_t(num_vectors)*num_features];
                 for (int i=0; i< num_vectors*num_features; ++i){
                     ar & feature_matrix[i];
                 }
@@ -715,186 +828,46 @@ template <class ST> class CSimpleFeatures: public CDotFeatures
 };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-/** get feature type the BOOL feature can deal with
- *
- * @return feature type BOOL
- */
-template<> inline EFeatureType CSimpleFeatures<bool>::get_feature_type()
-{
-	return F_BOOL;
+
+#define GET_FEATURE_TYPE(f_type, sg_type)	\
+template<> inline EFeatureType CSimpleFeatures<sg_type>::get_feature_type() \
+{ 																			\
+	return f_type; 															\
 }
 
-/** get feature type the CHAR feature can deal with
- *
- * @return feature type CHAR
- */
-template<> inline EFeatureType CSimpleFeatures<char>::get_feature_type()
-{
-	return F_CHAR;
+GET_FEATURE_TYPE(F_BOOL, bool)
+GET_FEATURE_TYPE(F_CHAR, char)
+GET_FEATURE_TYPE(F_BYTE, uint8_t)
+GET_FEATURE_TYPE(F_SHORT, int16_t)
+GET_FEATURE_TYPE(F_WORD, uint16_t)
+GET_FEATURE_TYPE(F_INT, int32_t)
+GET_FEATURE_TYPE(F_UINT, uint32_t)
+GET_FEATURE_TYPE(F_LONG, int64_t)
+GET_FEATURE_TYPE(F_ULONG, uint64_t)
+GET_FEATURE_TYPE(F_SHORTREAL, float32_t)
+GET_FEATURE_TYPE(F_DREAL, float64_t)
+GET_FEATURE_TYPE(F_LONGREAL, floatmax_t)
+#undef GET_FEATURE_TYPE
+
+#define GET_FEATURE_NAME(f_name, sg_type)									\
+template<> inline const char* CSimpleFeatures<sg_type>::get_name() const	\
+{ 																			\
+	return f_name; 															\
 }
 
-/** get feature type the BYTE feature can deal with
- *
- * @return feature type BYTE
- */
-template<> inline EFeatureType CSimpleFeatures<uint8_t>::get_feature_type()
-{
-	return F_BYTE;
-}
-
-/** get feature type the SHORT feature can deal with
- *
- * @return feature type SHORT
- */
-template<> inline EFeatureType CSimpleFeatures<int16_t>::get_feature_type()
-{
-	return F_SHORT;
-}
-
-/** get feature type the WORD feature can deal with
- *
- * @return feature type WORD
- */
-template<> inline EFeatureType CSimpleFeatures<uint16_t>::get_feature_type()
-{
-	return F_WORD;
-}
-
-
-/** get feature type the INT feature can deal with
- *
- * @return feature type INT
- */
-template<> inline EFeatureType CSimpleFeatures<int32_t>::get_feature_type()
-{
-	return F_INT;
-}
-
-/** get feature type the UINT feature can deal with
- *
- * @return feature type UINT
- */
-template<> inline EFeatureType CSimpleFeatures<uint32_t>::get_feature_type()
-{
-	return F_UINT;
-}
-
-/** get feature type the LONG feature can deal with
- *
- * @return feature type LONG
- */
-template<> inline EFeatureType CSimpleFeatures<int64_t>::get_feature_type()
-{
-	return F_LONG;
-}
-
-/** get feature type the ULONG feature can deal with
- *
- * @return feature type ULONG
- */
-template<> inline EFeatureType CSimpleFeatures<uint64_t>::get_feature_type()
-{
-	return F_ULONG;
-}
-
-/** get feature type the SHORTREAL feature can deal with
- *
- * @return feature type SHORTREAL
- */
-template<> inline EFeatureType CSimpleFeatures<float32_t>::get_feature_type()
-{
-	return F_SHORTREAL;
-}
-
-/** get feature type the DREAL feature can deal with
- *
- * @return feature type DREAL
- */
-template<> inline EFeatureType CSimpleFeatures<float64_t>::get_feature_type()
-{
-	return F_DREAL;
-}
-
-/** get feature type the LONGREAL feature can deal with
- *
- * @return feature type LONGREAL
- */
-template<> inline EFeatureType CSimpleFeatures<floatmax_t>::get_feature_type()
-{
-	return F_LONGREAL;
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<bool>::get_name() const
-{
-	return "BoolFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<char>::get_name() const
-{
-	return "CharFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<uint8_t>::get_name() const
-{
-	return "ByteFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<int16_t>::get_name() const
-{
-	return "ShortFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<uint16_t>::get_name() const
-{
-	return "WordFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<int32_t>::get_name() const
-{
-	return "IntFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<uint32_t>::get_name() const
-{
-	return "UIntFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<int64_t>::get_name() const
-{
-	return "LongIntFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<uint64_t>::get_name() const
-{
-	return "ULongIntFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<float32_t>::get_name() const
-{
-	return "ShortRealFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<float64_t>::get_name() const
-{
-	return "RealFeatures";
-}
-
-/** @return object name */
-template<> inline const char* CSimpleFeatures<floatmax_t>::get_name() const
-{
-	return "LongRealFeatures";
-}
+GET_FEATURE_NAME("BoolFeatures", bool)
+GET_FEATURE_NAME("CharFeatures", char)
+GET_FEATURE_NAME("ByteFeatures", uint8_t)
+GET_FEATURE_NAME("ShortFeatures", int16_t)
+GET_FEATURE_NAME("WordFeatures", uint16_t)
+GET_FEATURE_NAME("IntFeatures", int32_t)
+GET_FEATURE_NAME("UIntFeatures", uint32_t)
+GET_FEATURE_NAME("LongIntFeatures", int64_t)
+GET_FEATURE_NAME("ULongIntFeatures", uint64_t)
+GET_FEATURE_NAME("ShortRealFeatures", float32_t)
+GET_FEATURE_NAME("RealFeatures", float64_t)
+GET_FEATURE_NAME("LongRealFeatures", floatmax_t)
+#undef GET_FEATURE_NAME
 
 /** align strings and compute emperical kernel map based on alignment scores
  *
@@ -1162,6 +1135,53 @@ template<> inline float64_t CSimpleFeatures<floatmax_t>:: dense_dot(int32_t vec_
 
 	return result;
 }
+
+#define LOAD(f_load, sg_type)												\
+template<> inline void CSimpleFeatures<sg_type>::load(CFile* loader)		\
+{ 																			\
+	ASSERT(loader);															\
+	sg_type* matrix;														\
+	int32_t num_feat;														\
+	int32_t num_vec;														\
+	loader->f_load(matrix, num_feat, num_vec);								\
+	set_feature_matrix(matrix, num_feat, num_vec);							\
+}
+
+LOAD(get_bool_matrix, bool)
+LOAD(get_char_matrix, char)
+LOAD(get_byte_matrix, uint8_t)
+LOAD(get_short_matrix, int16_t)
+LOAD(get_word_matrix, uint16_t)
+LOAD(get_int_matrix, int32_t)
+LOAD(get_uint_matrix, uint32_t)
+LOAD(get_long_matrix, int64_t)
+LOAD(get_ulong_matrix, uint64_t)
+LOAD(get_shortreal_matrix, float32_t)
+LOAD(get_real_matrix, float64_t)
+LOAD(get_longreal_matrix, floatmax_t)
+#undef LOAD
+
+#define SAVE(f_write, sg_type)												\
+template<> inline void CSimpleFeatures<sg_type>::save(CFile* writer)		\
+{ 																			\
+	ASSERT(writer);															\
+	writer->f_write(feature_matrix, num_features, num_vectors);				\
+}
+
+SAVE(set_bool_matrix, bool)
+SAVE(set_char_matrix, char)
+SAVE(set_byte_matrix, uint8_t)
+SAVE(set_short_matrix, int16_t)
+SAVE(set_word_matrix, uint16_t)
+SAVE(set_int_matrix, int32_t)
+SAVE(set_uint_matrix, uint32_t)
+SAVE(set_long_matrix, int64_t)
+SAVE(set_ulong_matrix, uint64_t)
+SAVE(set_shortreal_matrix, float32_t)
+SAVE(set_real_matrix, float64_t)
+SAVE(set_longreal_matrix, floatmax_t)
+#undef SAVE
+
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 }
 #endif // _SIMPLEFEATURES__H__
