@@ -4,9 +4,10 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 1999-2009 Soeren Sonnenburg
+ * Written (W) 1999-2010 Soeren Sonnenburg
  * Written (W) 1999-2008 Gunnar Raetsch
  * Copyright (C) 1999-2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #ifndef _SPARSEFEATURES__H__
@@ -20,6 +21,7 @@
 #include "lib/Cache.h"
 #include "lib/io.h"
 #include "lib/Cache.h"
+#include "lib/File.h"
 
 #include "features/Labels.h"
 #include "features/Features.h"
@@ -30,6 +32,7 @@
 namespace shogun
 {
 
+class CFile;
 class CLabels;
 class CFeatures;
 class CDotFeatures;
@@ -145,15 +148,18 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 			}
 		}
 
-		/** constructor
+		/** constructor loading features from file
 		 *
-		 * @param fname filename to load features from
+		 * @param loader File object to load data from
 		 */
-		CSparseFeatures(char* fname)
-		: CDotFeatures(fname), num_vectors(0), num_features(0),
+		CSparseFeatures(CFile* loader)
+		: CDotFeatures(loader), num_vectors(0), num_features(0),
 			sparse_feature_matrix(NULL), feature_cache(NULL)
-		{}
+		{
+			load(loader);
+		}
 
+		/** default destructor */
 		virtual ~CSparseFeatures()
 		{
 			free_sparse_features();
@@ -379,7 +385,7 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		 * @param blen bvec's length
 		 * @return dot product between the two sparse feature vectors
 		 */
-		ST sparse_dot(ST alpha, TSparseEntry<ST>* avec, int32_t alen, TSparseEntry<ST>* bvec, int32_t blen)
+		static ST sparse_dot(ST alpha, TSparseEntry<ST>* avec, int32_t alen, TSparseEntry<ST>* bvec, int32_t blen)
 		{
 			ST result=0;
 
@@ -469,7 +475,11 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		void add_to_dense_vec(float64_t alpha, int32_t num, float64_t* vec, int32_t dim, bool abs_val=false)
 		{
 			ASSERT(vec);
-			ASSERT(dim==num_features);
+			if (dim!=num_features)
+			{
+				SG_ERROR("dimension of vec (=%d) does not match number of features (=%d)\n",
+						dim, num_features);
+			}
 
 			bool vfree;
 			int32_t num_feat;
@@ -555,6 +565,18 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 			}
 		}
 
+		/** get a transposed copy of the features
+		 *
+		 * @return transposed copy
+		 */
+		CSparseFeatures<ST>* get_transposed()
+		{
+			int32_t num_feat;
+			int32_t num_vec;
+			TSparse<ST>* s=get_transposed(num_feat, num_vec);
+			return new CSparseFeatures<ST>(s, num_feat, num_vec);
+		}
+
 		/** compute and return the transpose of the sparse feature matrix
 		 * which will be prepocessed.
 		 * num_feat, num_vectors are returned by reference
@@ -1014,6 +1036,18 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 			return CMath::abs(result);
 		}
 
+		/** load features from file
+		 *
+		 * @param loader File object to load data from
+		 */
+		void load(CFile* loader);
+
+		/** save features to file
+		 *
+		 * @param writer File object to write data to
+		 */
+		void save(CFile* writer);
+
 		/** load features from file
 		 *
 		 * @param fname filename to load from
@@ -1311,7 +1345,11 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		virtual float64_t dense_dot(int32_t vec_idx1, const float64_t* vec2, int32_t vec2_len)
 		{
 			ASSERT(vec2);
-			ASSERT(vec2_len==num_features);
+			if (vec2_len!=num_features)
+			{
+				SG_ERROR("dimension of vec2 (=%d) does not match number of features (=%d)\n",
+						vec2_len, num_features);
+			}
 			float64_t result=0;
 
 			bool vfree;
@@ -1329,6 +1367,96 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 			return result;
 		}
 
+		/** iterator for sparse features */
+		struct sparse_feature_iterator
+		{
+			/** pointer to feature vector */
+			TSparseEntry<ST>* sv;
+			/** index of vector */
+			int32_t vidx;
+			/** num */
+			int32_t num_feat_entries;
+			/** if we need to free the vector*/
+			bool vfree;
+
+			/** index */
+			int32_t index;
+
+			/** print details of iterator (for debugging purposes)*/
+			void print_info()
+			{
+				SG_SPRINT("sv=%p, vidx=%d, num_feat_entries=%d, index=%d\n",
+						sv, vidx, num_feat_entries, index);
+			}
+		};
+
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index)
+		{
+			if (vector_index>=num_vectors)
+			{
+				SG_ERROR("Index out of bounds (number of vectors %d, you "
+						"requested %d)\n", num_vectors, vector_index);
+			}
+
+			if (!sparse_feature_matrix)
+				SG_ERROR("Requires a in-memory feature matrix\n");
+
+			sparse_feature_iterator* it=new sparse_feature_iterator[1];
+			it->sv=get_sparse_feature_vector(vector_index, it->num_feat_entries, it->vfree);
+			it->vidx=vector_index;
+			it->index=0;
+
+			return it;
+		}
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator)
+		{
+			sparse_feature_iterator* it=(sparse_feature_iterator*) iterator;
+			if (!it || it->index >= it->num_feat_entries)
+				return false;
+
+			int32_t i=it->index++;
+
+			index =  it->sv[i].feat_index;
+			value = (float64_t) it->sv[i].entry;
+
+			return true;
+		}
+
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator)
+		{
+			if (!iterator)
+				return;
+
+			sparse_feature_iterator* it=(sparse_feature_iterator*) iterator;
+			free_sparse_feature_vector(it->sv, it->vidx, it->vfree);
+			delete[] it;
+		}
+
 		/** @return object name */
 		inline virtual const char* get_name() const { return "SparseFeatures"; }
 
@@ -1364,112 +1492,69 @@ template <class ST> class CSparseFeatures : public CDotFeatures
 		CCache< TSparseEntry<ST> >* feature_cache;
 };
 
-/** get feature type the BOOL feature can deal with
- *
- * @return feature type BOOL
- */
-template<> inline EFeatureType CSparseFeatures<bool>::get_feature_type()
-{
-	return F_BOOL;
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#define GET_FEATURE_TYPE(sg_type, f_type)									\
+template<> inline EFeatureType CSparseFeatures<sg_type>::get_feature_type()	\
+{																			\
+	return f_type;															\
 }
+GET_FEATURE_TYPE(bool, F_BOOL)
+GET_FEATURE_TYPE(char, F_CHAR)
+GET_FEATURE_TYPE(uint8_t, F_BYTE)
+GET_FEATURE_TYPE(int16_t, F_SHORT)
+GET_FEATURE_TYPE(uint16_t, F_WORD)
+GET_FEATURE_TYPE(int32_t, F_INT)
+GET_FEATURE_TYPE(uint32_t, F_UINT)
+GET_FEATURE_TYPE(int64_t, F_LONG)
+GET_FEATURE_TYPE(uint64_t, F_ULONG)
+GET_FEATURE_TYPE(float32_t, F_SHORTREAL)
+GET_FEATURE_TYPE(float64_t, F_DREAL)
+GET_FEATURE_TYPE(floatmax_t, F_LONGREAL)
+#undef GET_FEATURE_TYPE
 
-/** get feature type the CHAR feature can deal with
- *
- * @return feature type CHAR
- */
-template<> inline EFeatureType CSparseFeatures<char>::get_feature_type()
-{
-	return F_CHAR;
+#define LOAD(fname, sg_type)											\
+template<> inline void CSparseFeatures<sg_type>::load(CFile* loader)	\
+{																		\
+	ASSERT(loader);														\
+	TSparse<sg_type>* matrix=NULL;										\
+	int32_t num_feat=0;													\
+	int32_t num_vec=0;													\
+	loader->fname(matrix, num_feat, num_vec);			\
+	set_sparse_feature_matrix(matrix, num_feat, num_vec);				\
 }
+LOAD(get_bool_sparsematrix, bool)
+LOAD(get_char_sparsematrix, char)
+LOAD(get_byte_sparsematrix, uint8_t)
+LOAD(get_short_sparsematrix, int16_t)
+LOAD(get_word_sparsematrix, uint16_t)
+LOAD(get_int_sparsematrix, int32_t)
+LOAD(get_uint_sparsematrix, uint32_t)
+LOAD(get_long_sparsematrix, int64_t)
+LOAD(get_ulong_sparsematrix, uint64_t)
+LOAD(get_shortreal_sparsematrix, float32_t)
+LOAD(get_real_sparsematrix, float64_t)
+LOAD(get_longreal_sparsematrix, floatmax_t)
+#undef LOAD
 
-/** get feature type the BYTE feature can deal with
- *
- * @return feature type BYTE
- */
-template<> inline EFeatureType CSparseFeatures<uint8_t>::get_feature_type()
-{
-	return F_BYTE;
-}
-
-/** get feature type the SHORT feature can deal with
- *
- * @return feature type SHORT
- */
-template<> inline EFeatureType CSparseFeatures<int16_t>::get_feature_type()
-{
-	return F_SHORT;
-}
-
-/** get feature type the WORD feature can deal with
- *
- * @return feature type WORD
- */
-template<> inline EFeatureType CSparseFeatures<uint16_t>::get_feature_type()
-{
-	return F_WORD;
-}
-
-/** get feature type the INT feature can deal with
- *
- * @return feature type INT
- */
-template<> inline EFeatureType CSparseFeatures<int32_t>::get_feature_type()
-{
-	return F_INT;
-}
-
-/** get feature type the UINT feature can deal with
- *
- * @return feature type UINT
- */
-template<> inline EFeatureType CSparseFeatures<uint32_t>::get_feature_type()
-{
-	return F_UINT;
-}
-
-/** get feature type the LONG feature can deal with
- *
- * @return feature type LONG
- */
-template<> inline EFeatureType CSparseFeatures<int64_t>::get_feature_type()
-{
-	return F_LONG;
-}
-
-/** get feature type the ULONG feature can deal with
- *
- * @return feature type ULONG
- */
-template<> inline EFeatureType CSparseFeatures<uint64_t>::get_feature_type()
-{
-	return F_ULONG;
-}
-
-/** get feature type the SHORTREAL feature can deal with
- *
- * @return feature type SHORTREAL
- */
-template<> inline EFeatureType CSparseFeatures<float32_t>::get_feature_type()
-{
-	return F_SHORTREAL;
-}
-
-/** get feature type the DREAL feature can deal with
- *
- * @return feature type DREAL
- */
-template<> inline EFeatureType CSparseFeatures<float64_t>::get_feature_type()
-{
-	return F_DREAL;
-}
-
-/** get feature type the LONGREAL feature can deal with
- *
- * @return feature type LONGREAL
- */
-template<> inline EFeatureType CSparseFeatures<floatmax_t>::get_feature_type()
-{
-	return F_LONGREAL;
+#define WRITE(fname, sg_type)											\
+template<> inline void CSparseFeatures<sg_type>::save(CFile* writer)	\
+{																		\
+	ASSERT(writer);														\
+	writer->fname(sparse_feature_matrix, num_features, num_vectors);	\
 }
+WRITE(set_bool_sparsematrix, bool)
+WRITE(set_char_sparsematrix, char)
+WRITE(set_byte_sparsematrix, uint8_t)
+WRITE(set_short_sparsematrix, int16_t)
+WRITE(set_word_sparsematrix, uint16_t)
+WRITE(set_int_sparsematrix, int32_t)
+WRITE(set_uint_sparsematrix, uint32_t)
+WRITE(set_long_sparsematrix, int64_t)
+WRITE(set_ulong_sparsematrix, uint64_t)
+WRITE(set_shortreal_sparsematrix, float32_t)
+WRITE(set_real_sparsematrix, float64_t)
+WRITE(set_longreal_sparsematrix, floatmax_t)
+#undef WRITE
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 }
 #endif /* _SPARSEFEATURES__H__ */
