@@ -15,16 +15,17 @@
 #include "lib/common.h"
 #include "lib/io.h"
 #include "lib/Cache.h"
-#include "preproc/PreProc.h"
-#include "preproc/StringPreProc.h"
-#include "features/Features.h"
-#include "features/Alphabet.h"
 #include "lib/DynamicArray.h"
 #include "lib/File.h"
 #include "lib/MemoryMappedFile.h"
 #include "lib/Mathematics.h"
 #include "lib/Compressor.h"
 
+#include "preproc/PreProc.h"
+#include "preproc/StringPreProc.h"
+#include "features/Features.h"
+#include "features/Alphabet.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -173,6 +174,27 @@ template <class ST> class CStringFeatures : public CFeatures
 			set_features(p_features, p_num_vectors, p_max_string_length);
 		}
 
+		/** constructor
+		 *
+		 * @param p_features new features
+		 * @param p_num_vectors number of vectors
+		 * @param p_max_string_length maximum string length
+		 * @param alpha an actual alphabet
+		 */
+		CStringFeatures(T_STRING<ST>* p_features, int32_t p_num_vectors,
+				int32_t p_max_string_length, CAlphabet* alpha)
+		: CFeatures(0), num_vectors(0), features(NULL),
+			single_string(NULL),length_of_single_string(0),
+			max_string_length(0), order(0), symbol_mask_table(NULL),
+			preprocess_on_get(false), feature_cache(NULL)
+		{
+			alphabet=new CAlphabet(alpha);
+			SG_REF(alphabet);
+			num_symbols=alphabet->get_num_symbols();
+			original_num_symbols=num_symbols;
+			set_features(p_features, p_num_vectors, p_max_string_length);
+		}
+
 		/** constructor
 		 *
 		 * @param alpha alphabet to use for string features
@@ -213,7 +235,6 @@ template <class ST> class CStringFeatures : public CFeatures
 				for (int32_t i=0; i<num_vectors; i++)
 				{
 					features[i].string=new ST[orig.features[i].length];
-					ASSERT(features[i].string);
 					features[i].length=orig.features[i].length;
 					memcpy(features[i].string, orig.features[i].string, sizeof(ST)*orig.features[i].length);
 				}
@@ -229,11 +250,11 @@ template <class ST> class CStringFeatures : public CFeatures
 
 		/** constructor
 		 *
-		 * @param fname filename to load features from
+		 * @param loader File object via which to load data
 		 * @param alpha alphabet (type) to use for string features
 		 */
-		CStringFeatures(char* fname, EAlphabet alpha=DNA)
-		: CFeatures(fname), num_vectors(0), features(NULL), single_string(NULL),
+		CStringFeatures(CFile* loader, EAlphabet alpha=DNA)
+		: CFeatures(loader), num_vectors(0), features(NULL), single_string(NULL),
 			length_of_single_string(0), max_string_length(0), order(0),
 			symbol_mask_table(NULL), preprocess_on_get(false), feature_cache(NULL)
 		{
@@ -241,7 +262,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			SG_REF(alphabet);
 			num_symbols=alphabet->get_num_symbols();
 			original_num_symbols=num_symbols;
-			load(fname);
+			load(loader);
 		}
 
 		virtual ~CStringFeatures()
@@ -344,6 +365,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			ST* vec=get_feature_vector(num, l, free_vec);
 			*len=l;
 			*dst=(ST*) malloc(*len * sizeof(ST));
+			ASSERT(*dst);
 			memcpy(*dst, vec, *len * sizeof(ST));
 			free_feature_vector(vec, num, free_vec);
 		}
@@ -433,6 +455,61 @@ template <class ST> class CStringFeatures : public CFeatures
 			}
 		}
 
+		/** get a transposed copy of the features
+		 *
+		 * @return transposed copy
+		 */
+		CStringFeatures<ST>* get_transposed()
+		{
+			int32_t num_feat;
+			int32_t num_vec;
+			T_STRING<ST>* s=get_transposed(num_feat, num_vec);
+
+			return new CStringFeatures<ST>(s, num_vec, num_feat, alphabet);
+		}
+
+		/** compute and return the transpose of string features matrix
+		 * which will be prepocessed.
+		 * num_feat, num_vectors are returned by reference
+		 * caller has to clean up
+		 *
+		 * note that strings all have to have same length
+		 *
+		 * @param num_feat number of features in matrix
+		 * @param num_vec number of vectors in matrix
+		 * @return transposed string features
+		 */
+		T_STRING<ST>* get_transposed(int32_t &num_feat, int32_t &num_vec)
+		{
+			num_feat=num_vectors;
+			num_vec=get_max_vector_length();
+			ASSERT(have_same_length());
+
+			SG_DEBUG("Allocating memory for transposed string features of size %ld\n",
+					int64_t(num_feat)*num_vec);
+
+			T_STRING<ST>* sf=new T_STRING<ST>[num_vec];
+
+			for (int32_t i=0; i<num_vec; i++)
+			{
+				sf[i].string=new ST[num_feat];
+				sf[i].length=num_feat;
+			}
+
+			for (int32_t i=0; i<num_feat; i++)
+			{
+				int32_t len=0;
+				bool free_vec=false;
+				ST* vec=get_feature_vector(i, len, free_vec);
+
+				for (int32_t j=0; j<num_vec; j++)
+					sf[j].string[i]=vec[j];
+
+				free_feature_vector(vec, i, free_vec);
+			}
+			return sf;
+		}
+
 		/** free feature vector
 		 *
 		 * @param feat_vec feature vector to free
@@ -565,87 +642,31 @@ template <class ST> class CStringFeatures : public CFeatures
 
 		/** load features from file
 		 *
-		 * @param fname filename to load from
-		 * @return if loading was successful
+		 * @param loader File object via which to load data
 		 */
-		virtual bool load(char* fname)
-		{
-			SG_INFO( "loading...\n");
-			int64_t length=0;
-			max_string_length=0;
+		virtual inline void load(CFile* loader);
 
-			CFile f(fname, 'r', F_CHAR);
-			char* feature_matrix=f.load_char_data(NULL, length);
-
-			SG_DEBUG("char data now at %p of length %ld\n", 
-					feature_matrix, (int64_t) length);
-
-			num_vectors=0;
-
-			if (f.is_ok())
-			{
-				for (int64_t i=0; i<length; i++)
-				{
-					if (feature_matrix[i]=='\n')
-						num_vectors++;
-				}
-
-				SG_INFO( "file contains %ld vectors\n", num_vectors);
-				features= new T_STRING<ST>[num_vectors];
-
-				int64_t index=0;
-				for (int32_t lines=0; lines<num_vectors; lines++)
-				{
-					char* p=&feature_matrix[index];
-					int32_t columns=0;
-
-					for (columns=0; index+columns<length && p[columns]!='\n'; columns++);
-
-					if (index+columns>=length && p[columns]!='\n') {
-						SG_ERROR( "error in \"%s\":%d\n", fname, lines);
-					}
-
-					features[lines].length=columns;
-					features[lines].string=new ST[columns];
-
-					max_string_length=CMath::max(max_string_length,columns);
-
-					for (int32_t i=0; i<columns; i++)
-						features[lines].string[i]= ((ST) p[i]);
-
-					index+= features[lines].length+1;
-				}
-
-				num_symbols=4; //FIXME
-				return true;
-			}
-			else
-				SG_ERROR( "reading file failed\n");
-
-			return false;
-		}
-
-		/** load DNA features from file
+		/** load ascii line-based string features from file
 		 *
 		 * @param fname filename to load from
-		 * @param remap_to_bin if remap_to_bin
-		 * @return if loading was successful
+		 * @param remap_to_bin if translation to other binary alphabet
+		 * should be performed
+		 * @param ascii_alphabet src alphabet
+		 * @param binary_alphabet alphabet to translate to
 		 */
-		bool load_dna_file(char* fname, bool remap_to_bin=true)
+		void load_ascii_file(char* fname, bool remap_to_bin=true,
+				EAlphabet ascii_alphabet=DNA, EAlphabet binary_alphabet=RAWDNA)
 		{
-			bool result=false;
-
 			size_t blocksize=1024*1024;
 			size_t required_blocksize=0;
 			uint8_t* dummy=new uint8_t[blocksize];
 			uint8_t* overflow=NULL;
 			int32_t overflow_len=0;
 
-			num_symbols=4;
 			cleanup();
 
-			CAlphabet* alpha=new CAlphabet(DNA);
-			CAlphabet* alpha_bin=new CAlphabet(RAWDNA);
+			CAlphabet* alpha=new CAlphabet(ascii_alphabet);
+			CAlphabet* alpha_bin=new CAlphabet(binary_alphabet);
 
 			FILE* f=fopen(fname, "ro");
 
@@ -717,6 +738,7 @@ template <class ST> class CStringFeatures : public CFeatures
 									features[lines].string[j]=alpha->remap_to_bin(overflow[j]);
 								for (int32_t j=0; j<len; j++)
 									features[lines].string[j+overflow_len]=alpha->remap_to_bin(dummy[old_sz+j]);
+								alpha->add_string_to_histogram(&dummy[old_sz], len);
 								alpha_bin->add_string_to_histogram(features[lines].string, features[lines].length);
 							}
 							else
@@ -725,6 +747,7 @@ template <class ST> class CStringFeatures : public CFeatures
 									features[lines].string[j]=overflow[j];
 								for (int32_t j=0; j<len; j++)
 									features[lines].string[j+overflow_len]=dummy[old_sz+j];
+								alpha->add_string_to_histogram(&dummy[old_sz], len);
 								alpha->add_string_to_histogram(features[lines].string, features[lines].length);
 							}
 
@@ -742,13 +765,16 @@ template <class ST> class CStringFeatures : public CFeatures
 
 					overflow_len=sz-old_sz;
 				}
-				result=true;
-				SG_INFO("file successfully read\n");
-				SG_INFO("max_string_length=%d\n", max_string_length);
-				SG_INFO("num_strings=%d\n", num_vectors);
+
+				if (alpha->check_alphabet_size() && alpha->check_alphabet())
+				{
+					SG_INFO("file successfully read\n");
+					SG_INFO("max_string_length=%d\n", max_string_length);
+					SG_INFO("num_strings=%d\n", num_vectors);
+				}
+				fclose(f);
 			}
 
-			fclose(f);
 			delete[] dummy;
 
 			SG_UNREF(alphabet);
@@ -758,8 +784,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			else
 				alphabet = alpha;
 			SG_REF(alphabet);
-
-			return result;
+			num_symbols=alphabet->get_num_symbols();
 		}
 
 		/** load fasta file as string features
@@ -794,6 +819,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			cleanup();
 			SG_UNREF(alphabet);
 			alphabet=new CAlphabet(DNA);
+			num_symbols=alphabet->get_num_symbols();
 
 			T_STRING<ST>* strings=new T_STRING<ST>[num];
 			offs=0;
@@ -1079,6 +1105,87 @@ template <class ST> class CStringFeatures : public CFeatures
 			return false;
 		}
 
+		/** append features
+		 *
+		 * @param sf features to append
+		 * @return if setting was successful
+		 */
+		bool append_features(CStringFeatures<ST>* sf)
+		{
+			ASSERT(sf);
+			T_STRING<ST>* new_features = new T_STRING<ST>[sf->num_vectors];
+
+			for (int32_t i=0; i<sf->num_vectors; i++)
+			{
+				int32_t length=sf->features[i].length;
+				new_features[i].string=new ST[length];
+				memcpy(new_features[i].string, sf->features[i].string, length);
+				new_features[i].length=length;
+			}
+			return append_features(new_features, sf->num_vectors,
+					sf->max_string_length);
+		}
+
+		/** append features
+		 *
+		 * @param p_features features to append
+		 * @param p_num_vectors number of vectors
+		 * @param p_max_string_length maximum string length
+		 *
+		 * note that p_features will be delete[]'d on success
+		 *
+		 * @return if setting was successful
+		 */
+		bool append_features(T_STRING<ST>* p_features, int32_t p_num_vectors, int32_t p_max_string_length)
+        {
+            if (!features)
+                return set_features(p_features, p_num_vectors, p_max_string_length);
+
+            CAlphabet* alpha=new CAlphabet(alphabet->get_alphabet());
+
+            //compute histogram for char/byte
+            for (int32_t i=0; i<p_num_vectors; i++)
+                alpha->add_string_to_histogram( p_features[i].string, p_features[i].length);
+
+            SG_INFO("max_value_in_histogram:%d\n", alpha->get_max_value_in_histogram());
+            SG_INFO("num_symbols_in_histogram:%d\n", alpha->get_num_symbols_in_histogram());
+
+            if (alpha->check_alphabet_size() && alpha->check_alphabet())
+            {
+                SG_UNREF(alpha);
+                for (int32_t i=0; i<p_num_vectors; i++)
+                    alphabet->add_string_to_histogram( p_features[i].string, p_features[i].length);
+
+                int32_t old_num_vectors=num_vectors;
+                num_vectors=old_num_vectors+p_num_vectors;
+                T_STRING<ST>* new_features = new T_STRING<ST>[num_vectors];
+
+                for (int32_t i=0; i<num_vectors; i++)
+                {
+                    if (i<old_num_vectors)
+                    {
+                        new_features[i].string=features[i].string;
+                        new_features[i].length=features[i].length;
+                    }
+                    else
+                    {
+                        new_features[i].string=p_features[i-old_num_vectors].string;
+                        new_features[i].length=p_features[i-old_num_vectors].length;
+                    }
+                }
+                delete[] features;
+				delete[] p_features; // free now obsolete features
+
+                this->features=new_features;
+                this->max_string_length=CMath::max(max_string_length, p_max_string_length);
+
+                return true;
+            }
+            SG_UNREF(alpha);
+
+            return false;
+        }
+
 		/** get_features
 		 *
 		 * @param num_str number of strings (returned)
@@ -1106,7 +1213,7 @@ template <class ST> class CStringFeatures : public CFeatures
 			max_str_len=max_string_length;
 			T_STRING<ST>* new_feat=new T_STRING<ST>[num_str];
 
-			for (int i=0; i<num_str; i++)
+			for (int32_t i=0; i<num_str; i++)
 			{
 				int32_t len;
 				bool free_vec;
@@ -1135,13 +1242,9 @@ template <class ST> class CStringFeatures : public CFeatures
 
 		/** save features to file
 		 *
-		 * @param dest filename to save to
-		 * @return if saving was successful
+		 * @param writer File object via which to save data
 		 */
-		virtual bool save(char* dest)
-		{
-			return false;
-		}
+		virtual inline void save(CFile* writer);
 
 		/** load compressed features from file
 		 *
@@ -1692,6 +1795,13 @@ template <class ST> class CStringFeatures : public CFeatures
 				max_string_length=CMath::max(max_string_length, features[i].length);
 		}
 
+		/** get a zero terminated copy of the string
+		 *
+		 * @param str the string to copy
+		 * @return zero terminated copy of str
+		 *
+		 * note that this function is only sensible for character strings
+		 */
 		static ST* get_zero_terminated_string_copy(T_STRING<ST> str)
 		{
 			int32_t l=str.length;
@@ -1719,6 +1829,85 @@ template <class ST> class CStringFeatures : public CFeatures
 		}
 
 
+		/** compute histogram over strings
+		 */
+		virtual void get_histogram(float64_t** hist, int32_t* rows, int32_t* cols, bool normalize=true)
+		{
+			int32_t nsym=get_num_symbols();
+			int32_t slen=get_max_vector_length();
+			int64_t sz=int64_t(nsym)*slen*sizeof(float64_t);
+			float64_t* h= (float64_t*) malloc(sz);
+			ASSERT(h);
+			memset(h, 0, sz);
+
+			float64_t* h_normalizer=new float64_t[slen];
+			memset(h_normalizer, 0, slen*sizeof(float64_t));
+			int32_t num_str=get_num_vectors();
+			for (int32_t i=0; i<num_str; i++)
+			{
+				int32_t len;
+				bool free_vec;
+				ST* vec=get_feature_vector(i, len, free_vec);
+				for (int32_t j=0; j<len; j++)
+				{
+					h[int64_t(j)*nsym+alphabet->remap_to_bin(vec[j])]++;
+					h_normalizer[j]++;
+				}
+				free_feature_vector(vec, i, free_vec);
+			}
+
+			if (normalize)
+			{
+				for (int32_t i=0; i<slen; i++)
+				{
+					for (int32_t j=0; j<nsym; j++)
+					{
+						if (h_normalizer && h_normalizer[i])
+							h[int64_t(i)*nsym+j]/=h_normalizer[i];
+					}
+				}
+			}
+			delete[] h_normalizer;
+
+			*hist=h;
+			*rows=nsym;
+			*cols=slen;
+		}
+
+		/** create some random strings based on normalized histogram
+		 */
+		virtual void create_random(float64_t* hist, int32_t rows, int32_t cols, int32_t num_vec)
+		{
+			ASSERT(rows == get_num_symbols());
+			cleanup();
+			float64_t* randoms=new float64_t[cols];
+			T_STRING<ST>* sf=new T_STRING<ST>[num_vec];
+
+			for (int32_t i=0; i<num_vec; i++)
+			{
+				sf[i].string=new ST[cols];
+				sf[i].length=cols;
+
+				CMath::random_vector(randoms, cols, 0.0, 1.0);
+
+				for (int32_t j=0; j<cols; j++)
+				{
+					float64_t lik=hist[int64_t(j)*rows+0];
+
+					int32_t c;
+					for (c=0; c<rows-1; c++)
+					{
+						if (randoms[j]<=lik)
+							break;
+						lik+=hist[int64_t(j)*rows+c+1];
+					}
+					sf[i].string[j]=alphabet->remap_to_char(c);
+				}
+			}
+			delete[] randoms;
+			set_features(sf, num_vec, cols);
+		}
+
 		/** @return object name */
 		inline virtual const char* get_name() const { return "StringFeatures"; }
 
@@ -2087,6 +2276,52 @@ template<> 	inline void CStringFeatures<float64_t>::unembed_word(float64_t word,
 template<> 	inline void CStringFeatures<floatmax_t>::unembed_word(floatmax_t word, uint8_t* seq, int32_t len)
 {
 }
+#define LOAD(f_load, sg_type)												\
+template<> inline void CStringFeatures<sg_type>::load(CFile* loader)		\
+{																			\
+	SG_INFO( "loading...\n");												\
+																			\
+	T_STRING<sg_type>* strs;												\
+	int32_t num_str;														\
+	int32_t max_len;														\
+	loader->f_load(strs, num_str, max_len);									\
+	set_features(strs, num_str, max_len);									\
+}
+
+LOAD(get_bool_string_list, bool)
+LOAD(get_char_string_list, char)
+LOAD(get_byte_string_list, uint8_t)
+LOAD(get_short_string_list, int16_t)
+LOAD(get_word_string_list, uint16_t)
+LOAD(get_int_string_list, int32_t)
+LOAD(get_uint_string_list, uint32_t)
+LOAD(get_long_string_list, int64_t)
+LOAD(get_ulong_string_list, uint64_t)
+LOAD(get_shortreal_string_list, float32_t)
+LOAD(get_real_string_list, float64_t)
+LOAD(get_longreal_string_list, floatmax_t)
+#undef LOAD
+
+#define SAVE(f_write, sg_type)												\
+template<> inline void CStringFeatures<sg_type>::save(CFile* writer)		\
+{ 																			\
+	ASSERT(writer);															\
+	writer->f_write(features, num_vectors);									\
+}
+
+SAVE(set_bool_string_list, bool)
+SAVE(set_char_string_list, char)
+SAVE(set_byte_string_list, uint8_t)
+SAVE(set_short_string_list, int16_t)
+SAVE(set_word_string_list, uint16_t)
+SAVE(set_int_string_list, int32_t)
+SAVE(set_uint_string_list, uint32_t)
+SAVE(set_long_string_list, int64_t)
+SAVE(set_ulong_string_list, uint64_t)
+SAVE(set_shortreal_string_list, float32_t)
+SAVE(set_real_string_list, float64_t)
+SAVE(set_longreal_string_list, floatmax_t)
+#undef SAVE
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 }
 #endif // _CSTRINGFEATURES__H__
