@@ -29,6 +29,7 @@ CWDFeatures::CWDFeatures(CStringFeatures<uint8_t>* str,
 
 	degree=order;
 	from_degree=from_order;
+	wd_weights=NULL;
 	set_wd_weights();
 	set_normalization_const();
 
@@ -36,7 +37,8 @@ CWDFeatures::CWDFeatures(CStringFeatures<uint8_t>* str,
 
 CWDFeatures::CWDFeatures(const CWDFeatures& orig)
 	: CDotFeatures(orig), strings(orig.strings),
-	degree(orig.degree), from_degree(orig.from_degree)
+	degree(orig.degree), from_degree(orig.from_degree),
+	normalization_const(orig.normalization_const)
 {
 	SG_REF(strings);
 	string_length=strings->get_max_vector_length();
@@ -45,13 +47,14 @@ CWDFeatures::CWDFeatures(const CWDFeatures& orig)
 	alphabet_size=alpha->get_num_symbols();
 	SG_UNREF(alpha);
 
+	wd_weights=NULL;
 	set_wd_weights();
-	set_normalization_const();
 }
 
 CWDFeatures::~CWDFeatures()
 {
 	SG_UNREF(strings);
+	delete[] wd_weights;
 }
 
 float64_t CWDFeatures::dot(int32_t vec_idx1, int32_t vec_idx2)
@@ -77,7 +80,7 @@ float64_t CWDFeatures::dot(int32_t vec_idx1, int32_t vec_idx2)
 	}
 	strings->free_feature_vector(vec1, vec_idx1, free_vec1);
 	strings->free_feature_vector(vec2, vec_idx2, free_vec2);
-	return sum;
+	return sum/CMath::sq(normalization_const);
 }
 
 float64_t CWDFeatures::dense_dot(int32_t vec_idx1, const float64_t* vec2, int32_t vec2_len)
@@ -160,6 +163,7 @@ void CWDFeatures::add_to_dense_vec(float64_t alpha, int32_t vec_idx1, float64_t*
 void CWDFeatures::set_wd_weights()
 {
 	ASSERT(degree>0 && degree<=8);
+	delete[] wd_weights;
 	wd_weights=new float64_t[degree];
 	w_dim=0;
 
@@ -172,16 +176,97 @@ void CWDFeatures::set_wd_weights()
 }
 
 
-void CWDFeatures::set_normalization_const()
+void CWDFeatures::set_normalization_const(float64_t n)
 {
-	normalization_const=0;
-	for (int32_t i=0; i<degree; i++)
-		normalization_const+=(string_length-i)*wd_weights[i]*wd_weights[i];
+	if (n==0)
+	{
+		normalization_const=0;
+		for (int32_t i=0; i<degree; i++)
+			normalization_const+=(string_length-i)*wd_weights[i]*wd_weights[i];
+
+		normalization_const=CMath::sqrt(normalization_const);
+	}
+	else
+		normalization_const=n;
 
-	normalization_const=CMath::sqrt(normalization_const);
 	SG_DEBUG("normalization_const:%f\n", normalization_const);
 }
 
+void* CWDFeatures::get_feature_iterator(int32_t vector_index)
+{
+	if (vector_index>=num_strings)
+	{
+		SG_ERROR("Index out of bounds (number of strings %d, you "
+				"requested %d)\n", num_strings, vector_index);
+	}
+
+	wd_feature_iterator* it=new wd_feature_iterator[1];
+
+	it->lim=CMath::min(degree, string_length);
+	it->vec= strings->get_feature_vector(vector_index, it->vlen, it->vfree);
+	it->vidx=vector_index;
+
+	it->vec = strings->get_feature_vector(vector_index, it->vlen, it->vfree);
+	it->val=new int32_t[it->vlen];
+	CMath::fill_vector(it->val, it->vlen, 0);
+
+	it->asize=alphabet_size;
+	it->asizem1=1;
+	it->offs=0;
+	it->k=0;
+	it->i=0;
+	it->o=0;
+
+	return it;
+}
+
+bool CWDFeatures::get_next_feature(int32_t& index, float64_t& value, void* iterator)
+{
+	wd_feature_iterator* it=(wd_feature_iterator*) iterator;
+
+	if (it->i + it->k >= it->vlen)
+	{
+		if (it->k < it->lim-1)
+		{
+			it->offs+=it->asize*it->vlen;
+			it->asize*=alphabet_size;
+			it->asizem1*=alphabet_size;
+			it->k++;
+			it->i=0;
+			it->o=it->offs;
+		}
+		else
+			return false;
+	}
+
+	int32_t i=it->i;
+	int32_t k=it->k;
+#ifdef DEBUG_WDFEATURES
+	SG_PRINT("i=%d k=%d offs=%d o=%d asize=%d asizem1=%d\n", i, k, it->offs, it->o, it->asize, it->asizem1);
+#endif
+
+	it->val[i]+=it->asizem1*it->vec[i+k];
+	value=wd_weights[k]/normalization_const;
+	index=it->val[i]+it->o;
+#ifdef DEBUG_WDFEATURES
+	SG_PRINT("index=%d val=%f w_size=%d lim=%d vlen=%d\n", index, value, w_dim, it->lim, it->vlen);
+#endif
+
+	it->o+=it->asize;
+	it->i=i+1;
+
+	return true;
+}
+
+void CWDFeatures::free_feature_iterator(void* iterator)
+{
+	ASSERT(iterator);
+	wd_feature_iterator* it=(wd_feature_iterator*) iterator;
+	strings->free_feature_vector(it->vec, it->vidx, it->vfree);
+	delete[] it->val;
+	delete[] it;
+}
+
 CFeatures* CWDFeatures::duplicate() const
 {
 	return new CWDFeatures(*this);
