@@ -4,8 +4,9 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2009 Soeren Sonnenburg
+ * Written (W) 2009-2010 Soeren Sonnenburg
  * Copyright (C) 2009 Fraunhofer Institute FIRST and Max-Planck-Society
+ * Copyright (C) 2010 Berlin Institute of Technology
  */
 
 #ifndef _WDFEATURES_H___
@@ -87,9 +88,70 @@ class CWDFeatures : public CDotFeatures
 		 */
 		virtual inline int32_t get_nnz_features_for_vector(int32_t num)
 		{
-			return w_dim/alphabet_size;
+			int32_t vlen=-1;
+			bool free_vec;
+			uint8_t* vec=strings->get_feature_vector(num, vlen, free_vec);
+			strings->free_feature_vector(vec, num, free_vec);
+			return degree*vlen;
 		}
 
+		/** iterator for weighted spectrum features */
+		struct wd_feature_iterator
+		{
+			/** pointer to feature vector */
+			uint8_t* vec;
+			/** index of vector */
+			int32_t vidx;
+			/** length of vector */
+			int32_t vlen;
+			/** if we need to free the vector*/
+			bool vfree;
+
+			/** @name Internal Parameters
+			 * parameters of interal feature gen loop
+			 */
+			//@{
+			int32_t lim;
+			int32_t* val;
+			int32_t asize;
+			int32_t asizem1;
+			int32_t offs;
+			int32_t k;
+			int32_t i;
+			int32_t o;
+			//@}
+		};
+
+		/** iterate over the non-zero features
+		 *
+		 * call get_feature_iterator first, followed by get_next_feature and
+		 * free_feature_iterator to cleanup
+		 *
+		 * @param vector_index the index of the vector over whose components to
+		 * 			iterate over
+		 * @return feature iterator (to be passed to get_next_feature)
+		 */
+		virtual void* get_feature_iterator(int32_t vector_index);
+
+		/** iterate over the non-zero features
+		 *
+		 * call this function with the iterator returned by get_first_feature
+		 * and call free_feature_iterator to cleanup
+		 *
+		 * @param index is returned by reference (-1 when not available)
+		 * @param value is returned by reference
+		 * @param iterator as returned by get_first_feature
+		 * @return true if a new non-zero feature got returned
+		 */
+		virtual bool get_next_feature(int32_t& index, float64_t& value, void* iterator);
+
+		/** clean up iterator
+		 * call this function with the iterator returned by get_first_feature
+		 *
+		 * @param iterator as returned by get_first_feature
+		 */
+		virtual void free_feature_iterator(void* iterator);
+
 		/** duplicate feature object
 		 *
 		 * @return feature object
@@ -124,12 +186,32 @@ class CWDFeatures : public CDotFeatures
 			return sizeof(float64_t);
 		}
 
+		/** set normalization constant
+		 * @param n n=0 means automagic */
+		void set_normalization_const(float64_t n=0);
+
+		/** get normalization constant */
+		inline float64_t get_normalization_const()
+		{
+			return normalization_const;
+		}
+
 		/** @return object name */
 		inline virtual const char* get_name() const { return "WDFeatures"; }
 
-	protected:
-		/** set normalization constant */
-		void set_normalization_const();
+		/** set wd weights
+		 *
+		 * @param weights new weights
+		 * @param d degree
+		 * @return if setting was successful
+		 */
+		void set_wd_weights(float64_t* weights, int32_t d)
+		{
+			ASSERT(d==degree);
+
+			for (int32_t i=0; i<degree; i++)
+				wd_weights[i]=weights[i];
+		}
 
 		/** create wd kernel weighting heuristic */
 		void set_wd_weights();
@@ -138,9 +220,6 @@ class CWDFeatures : public CDotFeatures
 		/** stringfeatures the wdfeatures are based on*/
 		CStringFeatures<uint8_t>* strings;
 
-		/** normalization const */
-		float64_t normalization_const;
-
 		/** degree */
 		int32_t degree;
 		/** from degree */
@@ -155,6 +234,10 @@ class CWDFeatures : public CDotFeatures
 		int32_t w_dim;
 		/** wd weights */
 		float64_t* wd_weights;
+
+		/** normalization const */
+		float64_t normalization_const;
+
 };
 }
 #endif // _WDFEATURES_H___
