@@ -13,6 +13,8 @@
 #include "features/Features.h"
 #include "lib/io.h"
 
+using namespace shogun;
+
 CConstKernel::CConstKernel(float64_t c)
 : CKernel(0), const_value(c)
 {
@@ -33,13 +35,3 @@ bool CConstKernel::init(CFeatures* l, CFeatures* r)
 	CKernel::init(l, r);
 	return init_normalizer();
 }
-
-bool CConstKernel::load_init(FILE* src)
-{
-	return false;
-}
-
-bool CConstKernel::save_init(FILE* dest)
-{
-	return false;
-}
