@@ -32,6 +32,13 @@
 #include <pthread.h>
 #endif
 
+
+#ifdef HAVE_BOOST_SERIALIZATION
+#include <boost/serialization/export.hpp>
+BOOST_CLASS_EXPORT(shogun::CKernel);
+#endif //HAVE_BOOST_SERIALIZATION
+
+
 using namespace shogun;
 
 CKernel::CKernel()
@@ -154,6 +161,9 @@ bool CKernel::init(CFeatures* l, CFeatures* r)
 bool CKernel::set_normalizer(CKernelNormalizer* n)
 {
 	SG_REF(n);
+	if (lhs && rhs)
+		n->init(this);
+
 	SG_UNREF(normalizer);
 	normalizer=n;
 
@@ -178,39 +188,16 @@ void CKernel::cleanup()
 
 
 
-bool CKernel::load(char* fname)
+void CKernel::load(CFile* loader)
 {
-	return false;
 }
 
-bool CKernel::save(char* fname)
+void CKernel::save(CFile* writer)
 {
-	int32_t i=0;
-	int32_t num_left=get_num_vec_lhs();
-	int32_t num_right=rhs->get_num_vectors();
-	KERNELCACHE_IDX num_total=num_left*num_right;
-
-	CFile f(fname, 'w', F_DREAL);
-
-    for (int32_t l=0; l< (int32_t) num_left && f.is_ok(); l++)
-	{
-		for (int32_t r=0; r< (int32_t) num_right && f.is_ok(); r++)
-		{
-			 if (!(i % (num_total/200+1)))
-				SG_PROGRESS(i, 0, num_total-1);
-
-			float64_t k=kernel(l,r);
-			f.save_real_data(&k, 1);
-
-			i++;
-		}
-	}
-	SG_DONE();
-
-	if (f.is_ok())
-		SG_INFO( "kernel matrix of size %ld x %ld written (filesize: %ld)\n", num_left, num_right, num_total*sizeof(KERNELCACHE_ELEM));
-
-    return (f.is_ok());
+	int32_t m,n;
+	float64_t* km=get_kernel_matrix<float64_t>(m,n, NULL);
+	writer->set_real_matrix(km, m,n);
+	delete[] km;
 }
 
 void CKernel::remove_lhs_and_rhs()
@@ -301,6 +288,9 @@ void CKernel::list_kernel()
 		case K_WEIGHTEDDEGREEPOS:
 			SG_INFO( "K_WEIGHTEDDEGREEPOS ");
 			break;
+		case K_WEIGHTEDDEGREERBF:
+			SG_INFO( "K_WEIGHTEDDEGREERBF ");
+			break;
 		case K_WEIGHTEDCOMMWORDSTRING:
 			SG_INFO( "K_WEIGHTEDCOMMWORDSTRING ");
 			break;
@@ -316,6 +306,9 @@ void CKernel::list_kernel()
 		case K_COMMULONGSTRING:
 			SG_INFO( "K_COMMULONGSTRING ");
 			break;
+		case K_SPECTRUMMISMATCHRBF:
+			SG_INFO( "K_SPECTRUMMISMATCHRBF ");
+			break;
 		case K_COMBINED:
 			SG_INFO( "K_COMBINED ");
 			break;
