@@ -14,13 +14,16 @@
 
 #include "lib/common.h"
 #include "lib/Signal.h"
+#include "lib/File.h"
 #include "lib/Mathematics.h"
 #include "base/SGObject.h"
 #include "features/Features.h"
 #include "kernel/KernelNormalizer.h"
 
+
 namespace shogun
 {
+	class CFile;
 	class CFeatures;
 	class CKernelNormalizer;
 	enum EFeatureType;
@@ -50,6 +53,7 @@ enum EKernelType
 	K_GAUSSIAN = 30,
 	K_SPARSEGAUSSIAN = 31,
 	K_GAUSSIANSHIFT = 32,
+	K_GAUSSIANMATCH = 33,
 	K_HISTOGRAM = 40,
 	K_SALZBERG = 41,
 	K_LOCALITYIMPROVED = 50,
@@ -57,11 +61,13 @@ enum EKernelType
 	K_FIXEDDEGREE = 70,
 	K_WEIGHTEDDEGREE =    80,
 	K_WEIGHTEDDEGREEPOS = 81,
+	K_WEIGHTEDDEGREERBF = 82,
 	K_WEIGHTEDCOMMWORDSTRING = 90,
 	K_POLYMATCH = 100,
 	K_ALIGNMENT = 110,
 	K_COMMWORDSTRING = 120,
 	K_COMMULONGSTRING = 121,
+	K_SPECTRUMMISMATCHRBF = 122,
 	K_COMBINED = 140,
 	K_AUC = 150,
 	K_CUSTOM = 160,
@@ -145,6 +151,10 @@ class CKernel : public CSGObject
 	friend class CAvgDiagKernelNormalizer;
 	friend class CRidgeKernelNormalizer;
 	friend class CFirstElementKernelNormalizer;
+	friend class CMultitaskKernelNormalizer;
+	friend class CMultitaskKernelMklNormalizer;
+	friend class CMultitaskKernelMaskNormalizer;
+	friend class CMultitaskKernelMaskPairNormalizer;
 	friend class CTanimotoKernelNormalizer;
 	friend class CDiceKernelNormalizer;
 
@@ -338,17 +348,15 @@ class CKernel : public CSGObject
 
 		/** load the kernel matrix
 		 *
-		 * @param fname filename to load from
-		 * @return if loading was successful
+		 * @param loader File object via which to load data
 		 */
-		bool load(char* fname);
+		void load(CFile* loader);
 
 		/** save kernel matrix
 		 *
-		 * @param fname filename to save to
-		 * @return if saving was successful
+		 * @param writer File object via which to save data
 		 */
-		bool save(char* fname);
+		void save(CFile* writer);
 
 		/** get left-hand side of features used in kernel
 		 *
@@ -611,12 +619,11 @@ class CKernel : public CSGObject
 		 */
 		virtual float64_t compute(int32_t x, int32_t y)=0;
 
-
 		/** compute row start offset for parallel kernel matrix computation
 		 *
 		 * @param offs offset
 		 * @param n number of columns
-		 * @param symmetric whether matrix is symmetric 
+		 * @param symmetric whether matrix is symmetric
 		 */
 		int32_t compute_row_start(int64_t offs, int32_t n, bool symmetric)
 		{
@@ -630,7 +637,6 @@ class CKernel : public CSGObject
 			return i_start;
 		}
 
-
 		/** helper for computing the kernel matrix in a parallel way
 		 *
 		 * @param p thread parameters
