@@ -15,8 +15,16 @@
 
 namespace shogun
 {
+
+/** normalizer type */
+enum ENormalizerType
+{
+	N_REGULAR = 0,
+	N_MULTITASK = 1
+};
+
 class CKernel;
-/** @brief The class Kernel Normalizer defines a function to postprocess kernel values. 
+/** @brief The class Kernel Normalizer defines a function to post-process kernel values.
  *
  * Formally it defines f(.,.,.)
  *
@@ -42,8 +50,14 @@ class CKernel;
 class CKernelNormalizer : public CSGObject
 {
 	public:
-		/** default constructor */
-		CKernelNormalizer() { }
+
+		/** default constructor
+		 */
+		CKernelNormalizer()
+		{
+			m_type = N_REGULAR;
+		}
+
 		/** default destructor */
 		virtual ~CKernelNormalizer() { }
 
@@ -70,6 +84,25 @@ class CKernelNormalizer : public CSGObject
 		 * @param idx_rhs index of right hand side vector
 		 */
 		virtual float64_t normalize_rhs(float64_t value, int32_t idx_rhs)=0;
+
+		/** getter for normalizer type
+		 */
+		ENormalizerType get_normalizer_type()
+		{
+			return m_type;
+		}
+
+		/** setter for normalizer type
+		 *  @param type type of normalizer
+		 */
+		void set_normalizer_type(ENormalizerType type)
+		{
+			m_type = type;
+		}
+
+	protected:
+		/** normalizer type */
+		ENormalizerType m_type;
 };
 }
 #endif
