@@ -18,14 +18,14 @@
 using namespace shogun;
 
 CPolyMatchStringKernel::CPolyMatchStringKernel(int32_t size, int32_t d, bool i)
-: CStringKernel<char>(size), degree(d), inhomogene(i)
+: CStringKernel<char>(size), degree(d), inhomogene(i), rescaling(false)
 {
 	set_normalizer(new CSqrtDiagKernelNormalizer());
 }
 
 CPolyMatchStringKernel::CPolyMatchStringKernel(
 	CStringFeatures<char>* l, CStringFeatures<char>* r, int32_t d, bool i)
-: CStringKernel<char>(10), degree(d), inhomogene(i)
+: CStringKernel<char>(10), degree(d), inhomogene(i), rescaling(false)
 {
 	set_normalizer(new CSqrtDiagKernelNormalizer());
 	init(l, r);
@@ -61,7 +61,12 @@ float64_t CPolyMatchStringKernel::compute(int32_t idx_a, int32_t idx_b)
 		if (avec[i]==bvec[i])
 			sum++;
 	}
+	float64_t result = ((float64_t) sum);
+
+	if (rescaling)
+		result/=alen;
+
 	((CStringFeatures<char>*) lhs)->free_feature_vector(avec, idx_a, free_avec);
 	((CStringFeatures<char>*) rhs)->free_feature_vector(bvec, idx_b, free_bvec);
-	return CMath::pow((float64_t) sum, degree);
+	return CMath::pow(result , degree);
 }
