@@ -83,6 +83,20 @@ class CPolyMatchStringKernel: public CStringKernel<char>
 		 */
 		virtual const char* get_name() const { return "PolyMatchString"; }
 
+		/** enable rescaling by length of feature vector
+		 *
+		 * @param n true to enable
+		 */
+		void set_rescaling_enabled(bool n)
+		{
+			rescaling=n;
+		}
+
+		bool get_rescaling_enabled()
+		{
+			return rescaling;
+		}
+
 	protected:
 		/** compute kernel function for features a and b
 		 * idx_{a,b} denote the index of the feature vectors
@@ -99,6 +113,8 @@ class CPolyMatchStringKernel: public CStringKernel<char>
 		int32_t degree;
 		/** if kernel is inhomogeneous */
 		bool inhomogene;
+		/** true to rescale kernel with string length */
+		bool rescaling;
 };
 }
 #endif /* _POLYMATCHSTRINGKERNEL_H___ */
