@@ -17,7 +17,10 @@
 #include "kernel/WeightedDegreeStringKernel.h"
 #include "lib/Trie.h"
 
-class CSVM ;
+namespace shogun
+{
+
+class CSVM;
 
 /** @brief The Weighted Degree Position String kernel (Weighted Degree kernel
  * with shifts).
@@ -93,20 +96,6 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<char>
 		/** clean up kernel */
 		virtual void cleanup();
 
-		/** load kernel init_data
-		 *
-		 * @param src file to load from
-		 * @return if loading was successful
-		 */
-		bool load_init(FILE* src);
-
-		/** save kernel init_data
-		 *
-		 * @param dest file to save to
-		 * @return if saving was successful
-		 */
-		bool save_init(FILE* dest);
-
 		/** return what type of kernel we are
 		 *
 		 * @return kernel type WEIGHTEDDEGREEPOS
@@ -404,7 +393,7 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<char>
 		 * @param len number of position weights
 		 * @return if setting was successful
 		 */
-		virtual bool set_position_weights(float64_t* pws, int32_t len=0);
+		virtual bool set_position_weights(float64_t* pws, int32_t len);
 
 		/** set position weights for left-hand side
 		 *
@@ -775,4 +764,5 @@ class CWeightedDegreePositionStringKernel: public CStringKernel<char>
 		/** alphabet of features */
 		CAlphabet* alphabet;
 };
+}
 #endif /* _WEIGHTEDDEGREEPOSITIONSTRINGKERNEL_H__ */
