@@ -30,6 +30,7 @@
 BOOST_CLASS_EXPORT(shogun::CWeightedDegreeStringKernel);
 #endif //HAVE_BOOST_SERIALIZATION
 
+
 using namespace shogun;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -631,8 +632,8 @@ bool CWeightedDegreeStringKernel::set_wd_weights_by_type(EWDKernType p_type)
 bool CWeightedDegreeStringKernel::set_weights(
 	float64_t* ws, int32_t d, int32_t len)
 {
-	if (d!=degree || len<1)
-		SG_ERROR("Dimension mismatch (should be de(seq_length | 1) x degree)\n");
+	if (d!=degree || len<0)
+		SG_ERROR("WD: Dimension mismatch (should be (seq_length | 1) x degree) got (%d x %d)\n", len, degree);
 
 	length=len;
 
