@@ -47,6 +47,7 @@ class CTime : public CSGObject
 		clock_t cur_runtime_diff(bool verbose=false);
 
 		/** get time difference between start and NOW in seconds
+		 * (wall clock time)
 		 *
 		 * @param verbose if time difference shall be printed
 		 * @return time difference between start and NOW in seconds
@@ -61,6 +62,7 @@ class CTime : public CSGObject
 		float64_t start(bool verbose=false);
 
 		/** get time difference between start and NOW in seconds
+		 * (wall clock time)
 		 *
 		 * @param verbose if time difference shall be printed
 		 * @return time difference between start and NOW in seconds
@@ -84,11 +86,10 @@ class CTime : public CSGObject
 		 *
 		 * @return runtime
 		 */
-		static int64_t get_runtime()
+		static float64_t get_runtime()
 		{
 			clock_t start_runtime = clock();
-			return(
-				(int64_t) ((float64_t)start_runtime*100.0/(float64_t)CLOCKS_PER_SEC));
+			return ((float64_t) start_runtime)/CLOCKS_PER_SEC;
 		}
 
 		/** get current time in seconds
