@@ -13,11 +13,21 @@
 #include "lib/ShogunException.h"
 #include "lib/memory.h"
 #include "lib/common.h"
+#include "lib/Set.h"
 
+using namespace shogun;
+
+#ifdef TRACE_MEMORY_ALLOCS
+extern CSet<CMemoryBlock>* sg_mallocs;
+#endif
 
 void* operator new(size_t size) throw (std::bad_alloc)
 {
 	void *p=malloc(size);
+#ifdef TRACE_MEMORY_ALLOCS
+	if (sg_mallocs)
+		sg_mallocs->add(CMemoryBlock(p,size));
+#endif
 	if (!p)
 	{
 		const size_t buf_len=128;
@@ -35,6 +45,10 @@ void* operator new(size_t size) throw (std::bad_alloc)
 
 void operator delete(void *p)
 {
+#ifdef TRACE_MEMORY_ALLOCS
+	if (sg_mallocs)
+		sg_mallocs->remove(CMemoryBlock(p));
+#endif
 	if (p)
 		free(p);
 }
@@ -42,6 +56,10 @@ void operator delete(void *p)
 void* operator new[](size_t size)
 {
 	void *p=malloc(size);
+#ifdef TRACE_MEMORY_ALLOCS
+	if (sg_mallocs)
+		sg_mallocs->add(CMemoryBlock(p,size));
+#endif
 
 	if (!p)
 	{
@@ -60,6 +78,24 @@ void* operator new[](size_t size)
 
 void operator delete[](void *p)
 {
+#ifdef TRACE_MEMORY_ALLOCS
+	if (sg_mallocs)
+		sg_mallocs->remove(CMemoryBlock(p));
+#endif
 	if (p)
 		free(p);
 }
+
+#ifdef TRACE_MEMORY_ALLOCS
+void list_memory_allocs()
+{
+	if (sg_mallocs)
+	{
+		int32_t num=sg_mallocs->get_num_elements();
+		printf("%d Blocks are allocated:\n", num);
+
+		for (int32_t i=0; i<num; i++)
+			sg_mallocs->get_element(i).display();
+	}
+}
+#endif
