@@ -101,8 +101,7 @@ CDynProg::CDynProg(int32_t num_svms /*= 8 */)
 	  m_num_raw_data(0),
 	  
 	  m_long_transitions(true),
-	  m_long_transition_threshold(1000),
-	  m_long_transition_max(100000)
+	  m_long_transition_threshold(1000)
 {
 	trans_list_forward = NULL ;
 	trans_list_forward_cnt = NULL ;
@@ -821,6 +820,9 @@ void CDynProg::best_path_set_segment_loss(
 void CDynProg::best_path_set_segment_ids_mask(
 	int32_t* segment_ids, float64_t* segment_mask, int32_t m)
 {
+
+	if (m!=m_observation_matrix.get_dim2())
+		SG_ERROR("size of segment_ids or segment_mask (%i)  does not match the size of the feature matrix (%i)", m, m_observation_matrix.get_dim2());
 	int32_t max_id = 0;
 	for (int32_t i=1;i<m;i++)
 		max_id = CMath::max(max_id,segment_ids[i]);
@@ -910,248 +912,6 @@ void CDynProg::get_path_losses(float64_t** losses, int32_t* seq_len)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-float64_t CDynProg::best_path_no_b(
-	int32_t max_iter, int32_t &best_iter, int32_t *my_path)
-{
-	CArray2<T_STATES> psi(max_iter, m_N) ;
-   psi.set_name("psi");
-	CArray<float64_t>* delta = new CArray<float64_t>(m_N) ;
-	CArray<float64_t>* delta_new = new CArray<float64_t>(m_N) ;
-	
-	{ // initialization
-		for (int32_t i=0; i<m_N; i++)
-		{
-			delta->element(i) = get_p(i) ;
-			psi.element(0, i)= 0 ;
-		}
-	} 
-	
-	float64_t best_iter_prob = CMath::ALMOST_NEG_INFTY ;
-	best_iter = 0 ;
-	
-	// recursion
-	for (int32_t t=1; t<max_iter; t++)
-	{
-		CArray<float64_t>* dummy;
-		int32_t NN=m_N ;
-		for (int32_t j=0; j<NN; j++)
-		{
-			float64_t maxj = delta->element(0) + m_transition_matrix_a.element(0,j);
-			int32_t argmax=0;
-			
-			for (int32_t i=1; i<NN; i++)
-			{
-				float64_t temp = delta->element(i) + m_transition_matrix_a.element(i,j);
-				
-				if (temp>maxj)
-				{
-					maxj=temp;
-					argmax=i;
-				}
-			}
-			delta_new->element(j)=maxj ;
-			psi.element(t, j)=argmax ;
-		}
-		
-		dummy=delta;
-		delta=delta_new;
-		delta_new=dummy;	//switch delta/delta_new
-		
-		{ //termination
-			float64_t maxj=delta->element(0)+get_q(0);
-			int32_t argmax=0;
-			
-			for (int32_t i=1; i<m_N; i++)
-			{
-				float64_t temp=delta->element(i)+get_q(i);
-				
-				if (temp>maxj)
-				{
-					maxj=temp;
-					argmax=i;
-				}
-			}
-			//pat_prob=maxj;
-			
-			if (maxj>best_iter_prob)
-			{
-				my_path[t]=argmax;
-				best_iter=t ;
-				best_iter_prob = maxj ;
-			} ;
-		} ;
-	}
-
-	
-	{ //state sequence backtracking
-		for (int32_t t = best_iter; t>0; t--)
-		{
-			my_path[t-1]=psi.element(t, my_path[t]);
-		}
-	}
-
-	delete delta ;
-	delete delta_new ;
-	
-	return best_iter_prob ;
-}
-
-void CDynProg::best_path_no_b_trans(
-	int32_t max_iter, int32_t &max_best_iter, int16_t nbest,
-	float64_t *prob_nbest, int32_t *my_paths)
-{
-	//T_STATES *psi=new T_STATES[max_iter*m_N*nbest] ;
-	CArray3<T_STATES> psi(max_iter, m_N, nbest) ;
-   psi.set_name("psi");
-	CArray3<int16_t> ktable(max_iter, m_N, nbest) ;
-   ktable.set_name("ktable");
-	CArray2<int16_t> ktable_ends(max_iter, nbest) ;
-   ktable_ends.set_name("ktable_ends");
-
-	CArray<float64_t> tempvv(nbest*m_N) ;
-	CArray<int32_t> tempii(nbest*m_N) ;
-
-	CArray2<T_STATES> path_ends(max_iter, nbest) ;
-   path_ends.set_name("path_ends");
-	CArray2<float64_t> *delta=new CArray2<float64_t>(m_N, nbest) ;
-   delta->set_name("delta");
-	CArray2<float64_t> *delta_new=new CArray2<float64_t>(m_N, nbest) ;
-   delta_new->set_name("delta_new");
-	CArray2<float64_t> delta_end(max_iter, nbest) ;
-   delta_end.set_name("delta_end");
-
-	CArray2<int32_t> paths(max_iter, nbest) ;
-   paths.set_name("paths");
-	paths.set_array(my_paths, max_iter, nbest, false, false) ;
-
-	{ // initialization
-		for (T_STATES i=0; i<m_N; i++)
-		{
-			delta->element(i,0) = get_p(i) ;
-			for (int16_t k=1; k<nbest; k++)
-			{
-				delta->element(i,k)=-CMath::INFTY ;
-				ktable.element(0,i,k)=0 ;
-			}
-		}
-	}
-	
-	// recursion
-	for (int32_t t=1; t<max_iter; t++)
-	{
-		CArray2<float64_t>* dummy=NULL;
-
-		for (T_STATES j=0; j<m_N; j++)
-		{
-			const T_STATES num_elem   = trans_list_forward_cnt[j] ;
-			const T_STATES *elem_list = trans_list_forward[j] ;
-			const float64_t *elem_val = trans_list_forward_val[j] ;
-			
-			int32_t list_len=0 ;
-			for (int16_t diff=0; diff<nbest; diff++)
-			{
-				for (int32_t i=0; i<num_elem; i++)
-				{
-					T_STATES ii = elem_list[i] ;
-					
-					tempvv.element(list_len) = -(delta->element(ii,diff) + elem_val[i]) ;
-					tempii.element(list_len) = diff*m_N + ii ;
-					list_len++ ;
-				}
-			}
-			CMath::qsort_index(tempvv.get_array(), tempii.get_array(), list_len) ;
-			
-			for (int16_t k=0; k<nbest; k++)
-			{
-				if (k<list_len)
-				{
-					delta_new->element(j,k)  = -tempvv[k] ;
-					psi.element(t,j,k)      = (tempii[k]%m_N) ;
-					ktable.element(t,j,k)   = (tempii[k]-(tempii[k]%m_N))/m_N ;
-				}
-				else
-				{
-					delta_new->element(j,k)  = -CMath::INFTY ;
-					psi.element(t,j,k)      = 0 ;
-					ktable.element(t,j,k)   = 0 ;
-				}
-			}
-		}
-		
-		dummy=delta;
-		delta=delta_new;
-		delta_new=dummy;	//switch delta/delta_new
-		
-		{ //termination
-			int32_t list_len = 0 ;
-			for (int16_t diff=0; diff<nbest; diff++)
-			{
-				for (T_STATES i=0; i<m_N; i++)
-				{
-					tempvv.element(list_len) = -(delta->element(i,diff)+get_q(i));
-					tempii.element(list_len) = diff*m_N + i ;
-					list_len++ ;
-				}
-			}
-			CMath::qsort_index(tempvv.get_array(), tempii.get_array(), list_len) ;
-			
-			for (int16_t k=0; k<nbest; k++)
-			{
-				delta_end.element(t-1,k) = -tempvv[k] ;
-				path_ends.element(t-1,k) = (tempii[k]%m_N) ;
-				ktable_ends.element(t-1,k) = (tempii[k]-(tempii[k]%m_N))/m_N ;
-			}
-		}
-	}
-	
-	{ //state sequence backtracking
-		max_best_iter=0 ;
-		
-		CArray<float64_t> sort_delta_end(max_iter*nbest) ;
-		CArray<int16_t> sort_k(max_iter*nbest) ;
-		CArray<int32_t> sort_t(max_iter*nbest) ;
-		CArray<int32_t> sort_idx(max_iter*nbest) ;
-		
-		int32_t i=0 ;
-		for (int32_t iter=0; iter<max_iter-1; iter++)
-			for (int16_t k=0; k<nbest; k++)
-			{
-				sort_delta_end[i]=-delta_end.element(iter,k) ;
-				sort_k[i]=k ;
-				sort_t[i]=iter+1 ;
-				sort_idx[i]=i ;
-				i++ ;
-			}
-		
-		CMath::qsort_index(sort_delta_end.get_array(), sort_idx.get_array(), (max_iter-1)*nbest) ;
-
-		for (int16_t n=0; n<nbest; n++)
-		{
-			int16_t k=sort_k[sort_idx[n]] ;
-			int32_t iter=sort_t[sort_idx[n]] ;
-			prob_nbest[n]=-sort_delta_end[n] ;
-
-			if (iter>max_best_iter)
-				max_best_iter=iter ;
-			
-			ASSERT(k<nbest) ;
-			ASSERT(iter<max_iter) ;
-			
-			paths.element(iter,n) = path_ends.element(iter-1, k) ;
-			int16_t q   = ktable_ends.element(iter-1, k) ;
-			
-			for (int32_t t = iter; t>0; t--)
-			{
-				paths.element(t-1,n)=psi.element(t, paths.element(t,n), q);
-				q = ktable.element(t, paths.element(t,n), q) ;
-			}
-		}
-	}
-
-	delete delta ;
-	delete delta_new ;
-}
-
 bool CDynProg::extend_orf(
 	int32_t orf_from, int32_t orf_to, int32_t start, int32_t &last_pos,
 	int32_t to)
@@ -1386,7 +1146,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 							break ;
 					}
 			delete seq_input;
-			delete svm_value;
+			delete[] svm_value;
 		}
 
 		// allow longer transitions than look_back
@@ -1439,15 +1199,16 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 
 		CArray2<int32_t> look_back(m_N,m_N) ;
 		look_back.set_name("look_back");
-		CArray2<int32_t> look_back_orig(m_N,m_N) ;
-		look_back.set_name("look_back_orig");
+		//CArray2<int32_t> look_back_orig(m_N,m_N) ;
+		//look_back.set_name("look_back_orig");
+
 
 		{ // determine maximal length of look-back
 			for (int32_t i=0; i<m_N; i++)
 				for (int32_t j=0; j<m_N; j++)
 				{
 					look_back.set_element(INT_MAX, i, j) ;
-					look_back_orig.set_element(INT_MAX, i, j) ;
+					//look_back_orig.set_element(INT_MAX, i, j) ;
 				}
 
 			for (int32_t j=0; j<m_N; j++)
@@ -1466,7 +1227,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 						if (long_transitions)
 						{
 							look_back.set_element(m_long_transition_threshold, j, ii) ;
-							look_back_orig.set_element(m_long_transition_max, j, ii) ;
+							//look_back_orig.set_element(m_long_transition_max, j, ii) ;
 						}
 						continue ;
 					}
@@ -1475,7 +1236,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 					if ((m_orf_info.element(ii,0)!=-1) || (m_orf_info.element(j,1)!=-1) || (!long_transitions))
 					{
 						look_back.set_element(CMath::ceil(penij->get_max_value()), j, ii) ;
-						look_back_orig.set_element(CMath::ceil(penij->get_max_value()), j, ii) ;
+						//look_back_orig.set_element(CMath::ceil(penij->get_max_value()), j, ii) ;
 						if (CMath::ceil(penij->get_max_value()) > max_look_back)
 						{
 							SG_DEBUG( "%d %d -> value: %f\n", ii,j,penij->get_max_value());
@@ -1485,7 +1246,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 					else
 					{
 						look_back.set_element(CMath::min( (int32_t)CMath::ceil(penij->get_max_value()), m_long_transition_threshold ), j, ii) ;
-						look_back_orig.set_element( (int32_t)CMath::ceil(penij->get_max_value()), j, ii) ;
+						//look_back_orig.set_element( (int32_t)CMath::ceil(penij->get_max_value()), j, ii) ;
 					}
 					
 					if (penij->uses_svm_values())
@@ -1510,17 +1271,18 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 						if (long_transitions)
 						{
 							look_back.set_element(m_long_transition_threshold, i, j) ;
-							look_back_orig.set_element(m_long_transition_max, i, j) ;
+							//look_back_orig.set_element(m_long_transition_max, i, j) ;
 						}
 						else
 						{
 							look_back.set_element(max_look_back, i, j) ;
-							look_back_orig.set_element(m_long_transition_max, i, j) ;
+							//look_back_orig.set_element(m_long_transition_max, i, j) ;
 						}
 					}
 				}
 			SG_DEBUG("Using %i long transitions\n", num_long_transitions) ;
 		}
+		//SG_PRINT("max_look_back: %i \n", max_look_back) ;
 
 		//SG_PRINT("use_svm=%i, genestr_len: \n", use_svm, m_genestr.get_dim1()) ;
 		SG_DEBUG("use_svm=%i\n", use_svm) ;
@@ -1733,7 +1495,6 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 			}
 		}
 
-
 		SG_DEBUG("START_RECURSION \n\n");
 
 		// recursion
@@ -1743,7 +1504,6 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 			//	SG_PROGRESS(t, 0, m_seq_len);
 			//SG_PRINT("%i\n", t) ;
 
-
 			for (T_STATES j=0; j<m_N; j++)
 			{
 				if (seq.element(j,t)<=-1e20)
@@ -1804,9 +1564,6 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 						}
 
 						int32_t orf_last_pos = m_pos[t] ;
-						//int32_t loss_last_pos = t ;
-						//float64_t last_loss = 0.0 ;
-
 #ifdef DYNPROG_TIMING
 						MyTime3.start() ;
 #endif				
@@ -1961,7 +1718,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 						  } */
 
 						int32_t look_back_ = look_back.element(j, ii) ;
-						int32_t look_back_orig_ = look_back_orig.element(j, ii) ;
+						//int32_t look_back_orig_ = look_back_orig.element(j, ii) ;
 
 						int32_t orf_from = m_orf_info.element(ii,0) ;
 						int32_t orf_to   = m_orf_info.element(j,1) ;
@@ -1999,46 +1756,48 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 						if ( long_transitions && orf_target==-1 && look_back_ == m_long_transition_threshold )
 						{
 
-							int32_t start = long_transition_content_start.get_element(ii, j) ;
-							int32_t end_3p_part = start ;
-							for (int32_t start_3p_part=start; m_pos[t]-m_pos[start_3p_part] > m_long_transition_threshold ; start_3p_part++)
-							{
-								// find end_3p_part, which is greater than start_3p_part and at least m_long_transition_threshold away
-								while (end_3p_part<=t && m_pos[end_3p_part+1]-m_pos[start_3p_part]<=m_long_transition_threshold)
-									end_3p_part++ ;
+							// update table for 5' part  of the long segment
 
-								ASSERT(m_pos[end_3p_part+1]-m_pos[start_3p_part] > m_long_transition_threshold || end_3p_part==t) ;
-								ASSERT(m_pos[end_3p_part]-m_pos[start_3p_part] <= m_long_transition_threshold) ;
+							int32_t start = long_transition_content_start.get_element(ii, j) ;
+							int32_t end_5p_part = start ;
+							for (int32_t start_5p_part=start; m_pos[t]-m_pos[start_5p_part] > m_long_transition_threshold ; start_5p_part++)
+							{
+								// find end_5p_part, which is greater than start_5p_part and at least m_long_transition_threshold away
+								while (end_5p_part<=t && m_pos[end_5p_part+1]-m_pos[start_5p_part]<=m_long_transition_threshold)
+									end_5p_part++ ;
+
+								ASSERT(m_pos[end_5p_part+1]-m_pos[start_5p_part] > m_long_transition_threshold || end_5p_part==t) ;
+								ASSERT(m_pos[end_5p_part]-m_pos[start_5p_part] <= m_long_transition_threshold) ;
 
 								float64_t pen_val = 0.0;
 								/* recompute penalty, if necessary */
 								if (penalty)
 								{
 									int32_t frame = m_orf_info.element(ii,0);
-									lookup_content_svm_values(start_3p_part, end_3p_part, m_pos[start_3p_part], m_pos[end_3p_part], svm_value, frame); // * t -> end_3p_part 
-									pen_val = penalty->lookup_penalty(m_pos[end_3p_part]-m_pos[start_3p_part], svm_value) ;
+									lookup_content_svm_values(start_5p_part, end_5p_part, m_pos[start_5p_part], m_pos[end_5p_part], svm_value, frame); // * t -> end_5p_part 
+									pen_val = penalty->lookup_penalty(m_pos[end_5p_part]-m_pos[start_5p_part], svm_value) ;
 								}
 
-								/*if (m_pos[start_3p_part]==1003)
+								/*if (m_pos[start_5p_part]==1003)
 								  {
-								  SG_PRINT("Part1: %i - %i   vs  %i - %i\n", m_pos[t], m_pos[ts], m_pos[end_3p_part], m_pos[start_3p_part]) ;
-								  SG_PRINT("Part1: ts=%i  t=%i  start_3p_part=%i  m_seq_len=%i\n", m_pos[ts], m_pos[t], m_pos[start_3p_part], m_seq_len) ;
+								  SG_PRINT("Part1: %i - %i   vs  %i - %i\n", m_pos[t], m_pos[ts], m_pos[end_5p_part], m_pos[start_5p_part]) ;
+								  SG_PRINT("Part1: ts=%i  t=%i  start_5p_part=%i  m_seq_len=%i\n", m_pos[ts], m_pos[t], m_pos[start_5p_part], m_seq_len) ;
 								  }*/
 
-								float64_t mval_trans = -( elem_val[i] + pen_val*0.5 + delta.element(delta_array, start_3p_part, ii, 0, m_seq_len, m_N) ) ;
+								float64_t mval_trans = -( elem_val[i] + pen_val*0.5 + delta.element(delta_array, start_5p_part, ii, 0, m_seq_len, m_N) ) ;
 								//float64_t mval_trans = -( elem_val[i] + delta.element(delta_array, ts, ii, 0, m_seq_len, m_N) ) ; // enable this for the incomplete extra check
 
 								float64_t segment_loss_part1=0.0 ;
 								if (with_loss)
 								{  // this is the loss from the start of the long segment (5' part + middle section)
 
-									segment_loss_part1 = m_seg_loss_obj->get_segment_loss(start_3p_part /*long_transition_content_start_position.get_element(ii,j)*/, end_3p_part, elem_id[i]); // * unsure
+									segment_loss_part1 = m_seg_loss_obj->get_segment_loss(start_5p_part /*long_transition_content_start_position.get_element(ii,j)*/, end_5p_part, elem_id[i]); // * unsure
 
 									mval_trans -= segment_loss_part1 ;
 								}
 
 								
-								if (0)//m_pos[end_3p_part] - m_pos[long_transition_content_start_position.get_element(ii, j)] > look_back_orig_/*m_long_transition_max*/) // unsure: should it be end_3p_part or t ??
+								if (0)//m_pos[end_5p_part] - m_pos[long_transition_content_start_position.get_element(ii, j)] > look_back_orig_/*m_long_transition_max*/)
 								{
 									// this restricts the maximal length of segments, 
 									// but the current implementation is not valid since the 
@@ -2058,12 +1817,12 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 								if (with_loss)
 								{
 									float64_t old_loss = long_transition_content_scores_loss.get_element(ii, j) ;
-									float64_t new_loss = m_seg_loss_obj->get_segment_loss(long_transition_content_start_position.get_element(ii,j), end_3p_part, elem_id[i]);
+									float64_t new_loss = m_seg_loss_obj->get_segment_loss(long_transition_content_start_position.get_element(ii,j), end_5p_part, elem_id[i]);
 									float64_t score = long_transition_content_scores.get_element(ii, j) - old_loss + new_loss ;
 									long_transition_content_scores.set_element(score, ii, j) ;
 									long_transition_content_scores_loss.set_element(new_loss, ii, j) ;
 #ifdef DYNPROG_DEBUG
-									long_transition_content_end_position.set_element(end_3p_part, ii, j) ;
+									long_transition_content_end_position.set_element(end_5p_part, ii, j) ;
 #endif
 
 								}
@@ -2071,21 +1830,24 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 								{
 									/* then the old long transition is either too far away or worse than the current one */
 									long_transition_content_scores.set_element(-mval_trans, ii, j) ;
-									long_transition_content_start_position.set_element(start_3p_part, ii, j) ;
+									long_transition_content_start_position.set_element(start_5p_part, ii, j) ;
 									if (with_loss)
 										long_transition_content_scores_loss.set_element(segment_loss_part1, ii, j) ;
 #ifdef DYNPROG_DEBUG
 									long_transition_content_scores_pen.set_element(pen_val*0.5, ii, j) ;
 									long_transition_content_scores_elem.set_element(elem_val[i], ii, j) ;
-									long_transition_content_scores_prev.set_element(delta.element(delta_array, start_3p_part, ii, 0, m_seq_len, m_N), ii, j) ;
+									long_transition_content_scores_prev.set_element(delta.element(delta_array, start_5p_part, ii, 0, m_seq_len, m_N), ii, j) ;
 									/*ASSERT(fabs(long_transition_content_scores.get_element(ii, j)-(long_transition_content_scores_pen.get_element(ii, j) +
 									  long_transition_content_scores_elem.get_element(ii, j) + 
 									  long_transition_content_scores_prev.get_element(ii, j)))<1e-6) ;*/
-									long_transition_content_end_position.set_element(end_3p_part, ii, j) ;
+									long_transition_content_end_position.set_element(end_5p_part, ii, j) ;
 #endif
 								}
-
-								long_transition_content_start.set_element(start_3p_part, ii, j) ;
+								//
+								// this sets the position where the search for better 5'parts is started the next time
+								// whithout this the prediction takes ages
+								//
+								long_transition_content_start.set_element(start_5p_part, ii, j) ; 
 							}
 
 							// consider the 3' part at the end of the long segment:
@@ -2104,15 +1866,15 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 
 
 								/* only consider this transition, if the right position was found */
-								float pen_val = 0.0 ;
+								float pen_val_3p = 0.0 ;
 								if (penalty)
 								{
 									int32_t frame = orf_from ; //m_orf_info.element(ii, 0);
 									lookup_content_svm_values(ts, t, m_pos[ts], m_pos[t], svm_value, frame); 
-									pen_val = penalty->lookup_penalty(m_pos[t]-m_pos[ts], svm_value) ;
+									pen_val_3p = penalty->lookup_penalty(m_pos[t]-m_pos[ts], svm_value) ;
 								}
 
-								float64_t mval = -(long_transition_content_scores.get_element(ii, j) + pen_val*0.5) ;
+								float64_t mval = -(long_transition_content_scores.get_element(ii, j) + pen_val_3p*0.5) ;
 								
 								{
 #ifdef DYNPROG_DEBUG
@@ -2138,8 +1900,8 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 #ifdef DYNPROG_DEBUG
 									if (m_pos[t]==10108 ||m_pos[t]==12802 ||m_pos[t]== 12561)
 									{
-										SG_PRINT("Part2: %i,%i,%i: val=%1.6f  pen_val*0.5=%1.6f (t=%i, ts=%i, ts-1=%i, ts+1=%i); scores=%1.6f (pen=%1.6f,prev=%1.6f,elem=%1.6f,loss=%1.1f), positions=%i,%i,%i,  loss=%1.1f/%1.1f (%i,%i)\n", 
-												 m_pos[t], j, ii, -mval, 0.5*pen_val, m_pos[t], m_pos[ts], m_pos[ts-1], m_pos[ts+1], 
+										SG_PRINT("Part2: %i,%i,%i: val=%1.6f  pen_val_3p*0.5=%1.6f (t=%i, ts=%i, ts-1=%i, ts+1=%i); scores=%1.6f (pen=%1.6f,prev=%1.6f,elem=%1.6f,loss=%1.1f), positions=%i,%i,%i,  loss=%1.1f/%1.1f (%i,%i)\n", 
+												 m_pos[t], j, ii, -mval, 0.5*pen_val_3p, m_pos[t], m_pos[ts], m_pos[ts-1], m_pos[ts+1], 
 												 long_transition_content_scores.get_element(ii, j), 
 												 long_transition_content_scores_pen.get_element(ii, j), 
 												 long_transition_content_scores_prev.get_element(ii, j), 
@@ -2163,15 +1925,18 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 #endif
 								}
 
-
-								if ((mval < fixedtempvv_) &&
-									(m_pos[t] - m_pos[long_transition_content_start_position.get_element(ii, j)])<=look_back_orig_ /*m_long_transition_max*/)
+								// prefer simpler version to guarantee optimality
+								// 
+								// original:
+								/* if ((mval < fixedtempvv_) &&
+									(m_pos[t] - m_pos[long_transition_content_start_position.get_element(ii, j)])<=look_back_orig_) */
+								if (mval < fixedtempvv_)
 								{
 									/* then the long transition is better than the short one => replace it */ 
 									int32_t fromtjk =  fixedtempii_ ;
 									/*SG_PRINT("%i,%i: Long transition (%1.5f=-(%1.5f+%1.5f+%1.5f+%1.5f), %i) to m_pos %i better than short transition (%1.5f,%i) to m_pos %i \n", 
 									  m_pos[t], j, 
-									  mval, pen_val*0.5, long_transition_content_scores_pen.get_element(ii, j), long_transition_content_scores_elem.get_element(ii, j), long_transition_content_scores_prev.get_element(ii, j), ii, 
+									  mval, pen_val_3p*0.5, long_transition_content_scores_pen.get_element(ii, j), long_transition_content_scores_elem.get_element(ii, j), long_transition_content_scores_prev.get_element(ii, j), ii, 
 									  m_pos[long_transition_content_position.get_element(ii, j)], 
 									  fixedtempvv_, (fromtjk%m_N), m_pos[(fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)]) ;*/
 									ASSERT((fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)==0 || m_pos[(fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)]>=m_pos[long_transition_content_start_position.get_element(ii, j)] || fixedtemplong) ;
@@ -2183,7 +1948,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 								}
 
 								/* // extra check
-								   float64_t mval_trans2 = -( elem_val[i] + pen_val + delta.element(delta_array, ts, ii, 0, m_seq_len, m_N) ) ;
+								   float64_t mval_trans2 = -( elem_val[i] + pen_val_3p + delta.element(delta_array, ts, ii, 0, m_seq_len, m_N) ) ;
 								   if (last_ts==ts && fabs(last_mval-mval_trans2)>1e-5)
 								   SG_PRINT("last_mval=%1.2f at m_pos %i vs. mval_trans2=%1.2f at m_pos %i (diff=%f)\n", last_mval, m_pos[last_ts], mval_trans2, m_pos[ts], last_mval-mval_trans2) ;
 								   */
@@ -2234,7 +1999,6 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 				}
 			}
 		}
-
 		{ //termination
 			int32_t list_len = 0 ;
 			for (int16_t diff=0; diff<nbest; diff++)
@@ -2249,160 +2013,6 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 
 			CMath::nmin(oldtempvv.get_array(), oldtempii.get_array(), list_len, nbest) ;
 
-#ifdef nonsense
-			int t = m_seq_len-1;
-			for (T_STATES j=0; j<m_N; j++)
-			{
-				if (seq.element(j,t)<=-1e20)
-				{ // if we cannot observe the symbol here, then we can omit the rest
-					for (int16_t k=0; k<nbest; k++)
-					{
-						delta.element(delta_array, t, j, k, m_seq_len, m_N)    = seq.element(j,t) ;
-						psi.element(t,j,k)         = 0 ;
-						if (nbest>1)
-							ktable.element(t,j,k)  = 0 ;
-						ptable.element(t,j,k)      = 0 ;
-					}
-				}
-				else
-				{
-					const T_STATES num_elem   = trans_list_forward_cnt[j] ;
-					const T_STATES *elem_list = trans_list_forward[j] ;
-					const float64_t *elem_val      = trans_list_forward_val[j] ;
-					const int32_t *elem_id      = trans_list_forward_id[j] ;
-
-					int32_t fixed_list_len = 0 ;
-					float64_t fixedtempvv_ = CMath::INFTY ;
-					int32_t fixedtempii_ = 0 ;
-					bool fixedtemplong = false ;
-
-					for (int32_t i=0; i<num_elem; i++)
-					{
-						T_STATES ii = elem_list[i] ;
-
-						const CPlifBase * penalty = PEN.element(j,ii) ;
-
-						/*int32_t look_back = max_look_back ;
-						  if (0)
-						  { // find lookback length
-						  CPlifBase *pen = (CPlifBase*) penalty ;
-						  if (pen!=NULL)
-						  look_back=(int32_t) (CMath::ceil(pen->get_max_value()));
-						  if (look_back>=1e6)
-						  SG_PRINT("%i,%i -> %d from %ld\n", j, ii, look_back, (long)pen) ;
-						  ASSERT(look_back<1e6);
-						  } */
-
-						int32_t look_back_ = look_back.element(j, ii) ;
-						int32_t look_back_orig_ = look_back_orig.element(j, ii) ;
-
-						int32_t orf_from = m_orf_info.element(ii,0) ;
-						int32_t orf_to   = m_orf_info.element(j,1) ;
-						if((orf_from!=-1)!=(orf_to!=-1))
-							SG_DEBUG("j=%i  ii=%i  orf_from=%i orf_to=%i p=%1.2f\n", j, ii, orf_from, orf_to, elem_val[i]) ;
-						ASSERT((orf_from!=-1)==(orf_to!=-1)) ;
-
-						int32_t orf_target = -1 ;
-						if (orf_from!=-1)
-						{
-							orf_target=orf_to-orf_from ;
-							if (orf_target<0) 
-								orf_target+=3 ;
-							ASSERT(orf_target>=0 && orf_target<3) ;
-						}
-
-						int32_t orf_last_pos = m_pos[t] ;
-
-						int ts = t;
-						while (ts>0 && m_pos[t]-m_pos[ts-1] <= m_long_transition_threshold)
-							ts-- ;
-
-						if (ts>0)
-						{
-							ASSERT((m_pos[t]-m_pos[ts-1] > m_long_transition_threshold) && (m_pos[t]-m_pos[ts] <= m_long_transition_threshold)) ;
-
-							/* only consider this transition, if the right position was found */
-							float pen_val = 0.0 ;
-							if (penalty)
-							{
-								int32_t frame = orf_from ; //m_orf_info.element(ii, 0);
-								lookup_content_svm_values(ts, t, m_pos[ts], m_pos[t], svm_value, frame); 
-								pen_val = penalty->lookup_penalty(m_pos[t]-m_pos[ts], svm_value) ;
-							}
-
-							float64_t mval = -(long_transition_content_scores.get_element(ii, j) + pen_val*0.5) ;
-
-							{
-#ifdef DYNPROG_DEBUG
-								float64_t segment_loss_part2=0.0 ;
-								float64_t segment_loss_part1=0.0 ;
-#endif
-								float64_t segment_loss_total=0.0 ;
-
-								if (with_loss)
-								{   // this is the loss for the 3' end fragment of the segment
-									// (the 5' end and the middle section loss is already contained in mval)
-
-#ifdef DYNPROG_DEBUG
-									// this is an alternative, which should be identical, if the loss is additive
-									segment_loss_part2 = m_seg_loss_obj->get_segment_loss_extend(long_transition_content_end_position.get_element(ii,j), t, elem_id[i]);
-									//mval -= segment_loss_part2 ;
-									segment_loss_part1 = m_seg_loss_obj->get_segment_loss(long_transition_content_start_position.get_element(ii,j), long_transition_content_end_position.get_element(ii,j), elem_id[i]);
-#endif
-									segment_loss_total = m_seg_loss_obj->get_segment_loss(long_transition_content_start_position.get_element(ii,j), t, elem_id[i]);
-									mval -= (segment_loss_total-long_transition_content_scores_loss.get_element(ii, j)) ;
-								}
-
-#ifdef DYNPROG_DEBUG
-								if (1)//(m_pos[t]==10108 ||m_pos[t]==12802 ||m_pos[t]== 12561)
-								{
-									SG_PRINT("Part2: %i,%i,%i: val=%1.6f  pen_val*0.5=%1.6f (t=%i, ts=%i, ts-1=%i, ts+1=%i); scores=%1.6f (pen=%1.6f,prev=%1.6f,elem=%1.6f,loss=%1.1f), positions=%i,%i,%i,  loss=%1.1f/%1.1f (%i,%i)\n", 
-											m_pos[t], j, ii, -mval, 0.5*pen_val, m_pos[t], m_pos[ts], m_pos[ts-1], m_pos[ts+1], 
-											long_transition_content_scores.get_element(ii, j), 
-											long_transition_content_scores_pen.get_element(ii, j), 
-											long_transition_content_scores_prev.get_element(ii, j), 
-											long_transition_content_scores_elem.get_element(ii, j), 
-											long_transition_content_scores_loss.get_element(ii, j), 
-											m_pos[long_transition_content_start_position.get_element(ii,j)], 
-											m_pos[long_transition_content_end_position.get_element(ii,j)], 
-											m_pos[long_transition_content_start.get_element(ii,j)], segment_loss_part2, segment_loss_total, long_transition_content_start_position.get_element(ii,j), t) ;
-									SG_PRINT("oldtempvv: %1.6f, from_state:%i from_pos:%i\n ",-oldtempvv[0], (oldtempii[0]%m_N), m_pos[(oldtempii[0]-(oldtempii[0]%(m_N*nbest)))/(m_N*nbest)] );
-								}
-
-								if (fabs(segment_loss_part2+long_transition_content_scores_loss.get_element(ii, j) - segment_loss_total)>1e-3)
-								{
-									SG_ERROR("LOSS: total=%1.1f (%i-%i)  part1=%1.1f/%1.1f (%i-%i)  part2=%1.1f (%i-%i)  sum=%1.1f  diff=%1.1f\n", 
-											segment_loss_total, m_pos[long_transition_content_start_position.get_element(ii,j)], m_pos[t], 
-											long_transition_content_scores_loss.get_element(ii, j), segment_loss_part1, m_pos[long_transition_content_start_position.get_element(ii,j)], m_pos[long_transition_content_end_position.get_element(ii,j)],
-											segment_loss_part2, m_pos[long_transition_content_end_position.get_element(ii,j)], m_pos[t], 
-											segment_loss_part2+long_transition_content_scores_loss.get_element(ii, j), 
-											segment_loss_part2+long_transition_content_scores_loss.get_element(ii, j) - segment_loss_total) ;
-								}
-#endif
-							}
-
-
-							if ((mval < oldtempvv[0]) &&
-									(m_pos[t] - m_pos[long_transition_content_start_position.get_element(ii, j)])<=look_back_orig_ /*m_long_transition_max*/)
-							{
-								/* then the long transition is better than the short one => replace it */ 
-								int32_t fromtjk =  fixedtempii_ ;
-								/*SG_PRINT("%i,%i: Long transition (%1.5f=-(%1.5f+%1.5f+%1.5f+%1.5f), %i) to m_pos %i better than short transition (%1.5f,%i) to m_pos %i \n", 
-								  m_pos[t], j, 
-								  mval, pen_val*0.5, long_transition_content_scores_pen.get_element(ii, j), long_transition_content_scores_elem.get_element(ii, j), long_transition_content_scores_prev.get_element(ii, j), ii, 
-								  m_pos[long_transition_content_position.get_element(ii, j)], 
-								  fixedtempvv_, (fromtjk%m_N), m_pos[(fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)]) ;*/
-								ASSERT((fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)==0 || m_pos[(fromtjk-(fromtjk%(m_N*nbest)))/(m_N*nbest)]>=m_pos[long_transition_content_start_position.get_element(ii, j)] || fixedtemplong) ;
-
-								oldtempvv[0] = mval ;
-								oldtempii[0] = ii ;//+ m_N*long_transition_content_start_position.get_element(ii, j) ;
-							}
-						}
-
-					}
-				}
-			}
-#endif
 			for (int16_t k=0; k<nbest; k++)
 			{
 				delta_end.element(k) = -oldtempvv[k] ;
@@ -2466,6 +2076,7 @@ void CDynProg::compute_nbest_paths(int32_t max_num_signals, bool use_orf,
 		delete[] fixedtempii ;
 	}
 
+
 void CDynProg::best_path_trans_deriv(
 	int32_t *my_state_seq, int32_t *my_pos_seq,
 	int32_t my_seq_len, const float64_t *seq_array, int32_t max_num_signals)
@@ -2587,8 +2198,6 @@ void CDynProg::best_path_trans_deriv(
 		int32_t from_pos   = my_pos_seq[i] ;
 		int32_t to_pos     = my_pos_seq[i+1] ;
 
-		//int32_t loss_last_pos = to_pos ;
-		//float64_t last_loss = 0.0 ;
 		int32_t elem_id = m_transition_matrix_a_id.element(from_state, to_state) ;
 		my_losses[i] = m_seg_loss_obj->get_segment_loss(from_pos, to_pos, elem_id);
 
@@ -2637,6 +2246,7 @@ void CDynProg::best_path_trans_deriv(
 		{
 			if (is_long_transition)
 			{
+				
 				while (from_pos_thresh<to_pos && m_pos[from_pos_thresh+1] - m_pos[from_pos] <= m_long_transition_threshold) // *
 					from_pos_thresh++ ;
 				ASSERT(from_pos_thresh<to_pos) ;
@@ -2701,10 +2311,14 @@ void CDynProg::best_path_trans_deriv(
 				float64_t pen_value_part1 = PEN.element(to_state, from_state)->lookup_penalty(m_pos[from_pos_thresh]-m_pos[from_pos], svm_value_part1) ;
 				float64_t pen_value_part2 = PEN.element(to_state, from_state)->lookup_penalty(m_pos[to_pos]-m_pos[to_pos_thresh], svm_value_part2) ;
 				nscore= 0.5*pen_value_part1 + 0.5*pen_value_part2 ;
-				//fprintf(stdout, "pen_value_part1=%f  pen_value_part2=%f  nscore=%f\n", pen_value_part1, pen_value_part2, nscore) ;
 			}
 			else
 				nscore = PEN.element(to_state, from_state)->lookup_penalty(m_pos[to_pos]-m_pos[from_pos], svm_value) ;
+			
+			if (false)//(nscore<-1e9)
+					SG_PRINT("is_long_transition=%i  (from_pos=%i (%i), to_pos=%i (%i)=> %1.5f\n", 
+						is_long_transition, m_pos[from_pos], from_state, m_pos[to_pos], to_state, nscore) ;
+	
 			my_scores[i] += nscore ;
 
 			for (int32_t s=m_num_svms;s<m_num_lin_feat_plifs_cum[m_num_raw_data]; s++)/*set tiling plif values to neutral values (that do not influence derivative calculation)*/
@@ -2719,12 +2333,12 @@ void CDynProg::best_path_trans_deriv(
 #endif
 			if (is_long_transition)
 			{
+#ifdef DYNPROG_DEBUG
 				float64_t sum_score = 0.0 ;
 
 				for (int kk=0; kk<i; kk++)
 					sum_score += my_scores[i] ;
 
-#ifdef DYNPROG_DEBUG
 				SG_PRINT("is_long_transition=%i  (from_pos=%i (%i), to_pos=%i (%i)=> %1.5f, %1.5f --- 1: %1.6f (%i-%i)  2: %1.6f (%i-%i) \n", 
 						is_long_transition, m_pos[from_pos], from_state, m_pos[to_pos], to_state, 
 						nscore, sum_score, 
@@ -2819,6 +2433,20 @@ void CDynProg::best_path_trans_deriv(
 			{
 				float64_t nscore = PEN_state_signals.element(to_state,k)->lookup_penalty(seq_input.element(to_state, to_pos, k), svm_value) ; // this should be ok for long_transitions (svm_value does not matter)
 				my_scores[i] += nscore ;
+#ifdef DYNPROG_DEBUG
+				if (false)//(nscore<-1e9)
+				{
+					SG_PRINT("is_long_transition=%i  (from_pos=%i (%i), from_state=%i, to_pos=%i (%i) to_state=%i=> %1.5f, dim3:%i, seq_input.element(to_state, to_pos, k): %1.4f\n", 
+						is_long_transition, m_pos[from_pos], from_pos, from_state, m_pos[to_pos], to_pos, to_state, nscore, k, seq_input.element(to_state, to_pos, k)) ;
+					for (int x=0; x<23; x++)
+					{
+						for (int i=-10; i<10; i++)
+							SG_PRINT("%1.4f\t", seq_input.element(x, to_pos+i, k));
+						SG_PRINT("\n");
+					}
+					
+				}
+#endif
 				//break ;
 				//int32_t num_current_svms=0;
 				//int32_t svm_ids[] = {-8, -7, -6, -5, -4, -3, -2, -1};
