@@ -79,6 +79,7 @@ CGUIClassifier::CGUIClassifier(CSGInterface* ui_)
 	svm_bufsize=3000;
 	svm_max_qpsize=1000;
 	mkl_norm=1;
+	ent_lambda=0;
 	svm_C1=1;
 	svm_C2=1;
 	C_mkl=0;
@@ -203,23 +204,38 @@ bool CGUIClassifier::new_classifier(char* name, int32_t d, int32_t from_d)
 		SG_INFO("created Perceptron object\n") ;
 	}
 #ifdef HAVE_LAPACK
-	else if (strcmp(name,"LIBLINEAR_LR")==0)
+	else if (strncmp(name,"LIBLINEAR",9)==0)
 	{
+		LIBLINEAR_SOLVER_TYPE st=L2R_LR;
+		
+		if (strcmp(name,"LIBLINEAR_L2R_LR")==0)
+		{
+			st=L2R_LR;
+			SG_INFO("created LibLinear l2 regularized logistic regression object\n") ;
+		}
+		else if (strcmp(name,"LIBLINEAR_L2R_L2LOSS_SVC_DUAL")==0)
+		{
+			st=L2R_L2LOSS_SVC_DUAL;
+			SG_INFO("created LibLinear l2 regularized l2 loss SVM dual object\n") ;
+		}
+		else if (strcmp(name,"LIBLINEAR_L2R_L2LOSS_SVC")==0)
+		{
+			st=L2R_L2LOSS_SVC;
+			SG_INFO("created LibLinear l2 regularized l2 loss SVM primal object\n") ;
+		}
+		else if (strcmp(name,"LIBLINEAR_L2R_L1LOSS_SVC_DUAL")==0)
+		{
+			st=L2R_L1LOSS_SVC_DUAL;
+			SG_INFO("created LibLinear l2 regularized l1 loss dual SVM object\n") ;
+		}
+		else
+			SG_ERROR("unknown liblinear type\n");
+
 		SG_UNREF(classifier);
-		classifier= new CLibLinear(LR);
+		classifier= new CLibLinear(st);
 		((CLibLinear*) classifier)->set_C(svm_C1, svm_C2);
 		((CLibLinear*) classifier)->set_epsilon(svm_epsilon);
 		((CLibLinear*) classifier)->set_bias_enabled(svm_use_bias);
-		SG_INFO("created LibLinear logistic regression object\n") ;
-	}
-	else if (strcmp(name,"LIBLINEAR_L2")==0)
-	{
-		SG_UNREF(classifier);
-		classifier= new CLibLinear(L2);
-		((CLibLinear*) classifier)->set_C(svm_C1, svm_C2);
-		((CLibLinear*) classifier)->set_epsilon(svm_epsilon);
-		((CLibLinear*) classifier)->set_bias_enabled(svm_use_bias);
-		SG_INFO("created LibLinear l2 loss object\n") ;
 	}
 	else if (strcmp(name,"LDA")==0)
 	{
@@ -405,6 +421,7 @@ bool CGUIClassifier::train_mkl_multiclass()
 	SG_INFO("Starting MC-MKL training on %ld vectors using C1=%lf C2=%lf epsilon=%lf\n", num_vec, svm_C1, svm_C2, svm_epsilon);
 
 	mkl->set_mkl_epsilon(svm_weight_epsilon);
+	mkl->set_mkl_norm(mkl_norm); 
 	//mkl->set_max_num_mkliters(-1);
 	mkl->set_solver_type(solver_type);
 	mkl->set_bias_enabled(svm_use_bias);
@@ -468,6 +485,7 @@ bool CGUIClassifier::train_mkl()
 	mkl->set_batch_computation_enabled(svm_use_batch_computation);
 	mkl->set_mkl_epsilon(svm_weight_epsilon);
 	mkl->set_mkl_norm(mkl_norm); 
+	mkl->set_elasticnet_lambda(ent_lambda);
 	mkl->set_C_mkl(C_mkl);
 	mkl->set_interleaved_optimization_enabled(mkl_use_interleaved);
 
@@ -940,8 +958,8 @@ bool CGUIClassifier::set_svm_mkl_parameters(
 		weight_epsilon=1e-4;
 	if (C<0)
 		C=0;
-	if (norm<=0)
-		SG_ERROR("MKL norm > 0\n");
+	if (norm<0)
+		SG_ERROR("MKL norm >= 0\n");
 
 	svm_weight_epsilon=weight_epsilon;
 	C_mkl=C;
@@ -954,6 +972,15 @@ bool CGUIClassifier::set_svm_mkl_parameters(
 	return true;
 }
 
+bool CGUIClassifier::set_elasticnet_lambda(float64_t lambda)
+{
+  if (lambda<0 || lambda>1) {
+    SG_ERROR("0 <= ent_lambda <= 1\n");
+  }
+  ent_lambda = lambda;
+  return true;
+}
+
 bool CGUIClassifier::set_svm_C(float64_t C1, float64_t C2)
 {
 	if (C1<0)
@@ -1491,6 +1518,11 @@ bool CGUIClassifier::set_solver(char* solver)
 		SG_INFO("Using DIRECT solver\n");
 		s=ST_DIRECT;
 	}
+	else if (strncmp(solver,"ELASTICNET", 10)==0)
+	{
+		SG_INFO("Using ELASTICNET solver\n");
+		s=ST_ELASTICNET;
+	}
 	else if (strncmp(solver,"AUTO", 4)==0)
 	{
 		SG_INFO("Automagically determining solver.\n");
