@@ -59,6 +59,14 @@
 #define N_SET_HMM							"set_hmm"
 #define N_MODEL_PROB_NO_B_TRANS				"model_prob_no_b_trans"
 #define N_BEST_PATH_NO_B_TRANS				"best_path_no_b_trans"
+#define N_SIGNALS_SET_MODEL				"signals_set_model"
+#define N_SIGNALS_SET_POSITIONS 			"signals_set_positions"
+#define N_SIGNALS_SET_LABELS 				"signals_set_labels"
+#define N_SIGNALS_SET_SPLIT				"signals_set_split"
+#define N_SIGNALS_SET_TRAIN_MASK			"signals_set_train_mask"
+#define N_SIGNALS_ADD_FEATURE				"signals_add_feature"
+#define N_SIGNALS_ADD_KERNEL				"signals_add_kernel"
+#define N_SIGNALS_RUN					"signals_run"
 #define N_BEST_PATH_TRANS_DERIV				"best_path_trans_deriv"
 #define N_BEST_PATH_TRANS					"best_path_trans"
 #define N_SET_PLIF_STRUCT					"set_plif_struct"
@@ -69,6 +77,7 @@
 #define N_INIT_INTRON_LIST				"init_intron_list"
 #define N_SET_LIN_FEAT					"set_lin_feat"
 #define N_INIT_DYN_PROG					"init_dyn_prog"
+#define N_CLEAN_UP_DYN_PROG					"clean_up_dyn_prog"
 #define N_PRECOMPUTE_TILING_FEATURES			"precompute_tiling_features"
 #define N_LONG_TRANSITION_SETTINGS			"long_transition_settings"
 #define N_SET_MODEL					"set_model"
@@ -175,6 +184,7 @@
 #define N_SVMMAXQPSIZE					"svm_max_qpsize"
 #define N_THREADS						"threads"
 #define N_MKL_PARAMETERS				"mkl_parameters"
+#define N_ENT_LAMBDA                                    "elasticnet_lambda"
 #define N_SVM_EPSILON					"svm_epsilon"
 #define N_SVM_MAX_TRAIN_TIME			"svm_max_train_time"
 #define N_SVR_TUBE_EPSILON				"svr_tube_epsilon"
