@@ -15,6 +15,7 @@
 
 #include <shogun/lib/config.h>
 #include <shogun/lib/io.h>
+#include <shogun/lib/AsciiFile.h>
 #include <shogun/kernel/Kernel.h>
 #include <shogun/kernel/CombinedKernel.h>
 #include <shogun/kernel/Chi2Kernel.h>
@@ -23,6 +24,8 @@
 #include <shogun/kernel/LinearStringKernel.h>
 #include <shogun/kernel/LinearWordKernel.h>
 #include <shogun/kernel/WeightedDegreeStringKernel.h>
+#include <shogun/kernel/WeightedDegreeRBFKernel.h>
+#include <shogun/kernel/SpectrumMismatchRBFKernel.h>
 #include <shogun/kernel/WeightedDegreePositionStringKernel.h>
 #include <shogun/kernel/FixedDegreeStringKernel.h>
 #include <shogun/kernel/LocalityImprovedStringKernel.h>
@@ -55,6 +58,7 @@
 #include <shogun/kernel/IdentityKernelNormalizer.h>
 #include <shogun/kernel/SqrtDiagKernelNormalizer.h>
 #include <shogun/kernel/VarianceKernelNormalizer.h>
+#include <shogun/kernel/ScatterKernelNormalizer.h>
 #include <shogun/classifier/svm/SVM.h>
 
 #include <string.h>
@@ -371,6 +375,30 @@ float64_t* CGUIKernel::get_weights(int32_t order, int32_t max_mismatch)
 	return weights;
 }
 
+CKernel* CGUIKernel::create_weighteddegreerbf(int32_t size, int32_t degree, int32_t nof_properties, float64_t width)
+{
+	CKernel* kern=new CWeightedDegreeRBFKernel(size, width, degree, nof_properties);
+	if (!kern)
+		SG_ERROR("Couldn't create WeightedDegreeRBFKernel with size %d, width %f, degree %d, nof_properties %d.\n", size, width, degree, nof_properties);
+	else
+		SG_DEBUG("created WeightedDegreeRBFKernel (%p) with size %d, width %f, degree %d, nof_properties %d.\n", kern, size, width, degree, nof_properties);
+
+	return kern;
+}
+
+CKernel* CGUIKernel::create_spectrummismatchrbf(int32_t size, float64_t* AA_matrix, int32_t max_mismatch, int32_t degree, float64_t width)
+{
+
+	CKernel* kern = new CSpectrumMismatchRBFKernel(size, AA_matrix, degree, max_mismatch, width);
+	if (!kern)
+		SG_ERROR("Couldn't create SpectrumMismatchRBFKernel with size %d, width %f, degree %d, max_mismatch %d.\n", size, width, degree, max_mismatch);
+	else
+		SG_DEBUG("created SpectrumMismatchRBFKernel (%p) with size %d, width %f, degree %d, max_mismatch %d.\n", kern, size, width, degree, max_mismatch);
+
+	return kern;
+
+}
+
 
 CKernel* CGUIKernel::create_localalignmentstring(int32_t size)
 {
@@ -647,6 +675,13 @@ bool CGUIKernel::set_normalization(char* normalization, float64_t c, float64_t r
 		SG_INFO("Variance Normalization selected\n");
 		return k->set_normalizer(new CVarianceKernelNormalizer());
 	}
+	else if (strncmp(normalization,"SCATTER", 7)==0)
+	{
+		SG_INFO("Scatter Normalization selected\n");
+		CLabels* train_labels=ui->ui_labels->get_train_labels();
+		ASSERT(train_labels);
+		return k->set_normalizer(new CScatterKernelNormalizer(c,r, train_labels));
+	}
 	else
 		SG_ERROR("Wrong kernel normalizer name.\n");
 
@@ -720,7 +755,10 @@ bool CGUIKernel::init_kernel(const char* target)
 
 	// no need to init custom kernel
 	if (kernel->get_kernel_type() == K_CUSTOM || !target)
+	{
+		initialized=true;
 		return true;
+	}
 
 	EFeatureClass k_fclass=kernel->get_feature_class();
 	EFeatureType k_ftype=kernel->get_feature_type();
@@ -737,6 +775,7 @@ bool CGUIKernel::init_kernel(const char* target)
 				(k_ftype==ftype || k_ftype==F_ANY || ftype==F_ANY))
 			
 			{
+				SG_INFO("Initialising kernel with TRAIN DATA, train: %p\n", train);
 				kernel->init(train, train);
 				initialized=true;
 			}
@@ -750,7 +789,7 @@ bool CGUIKernel::init_kernel(const char* target)
 	{
 		CFeatures* train=ui->ui_features->get_train_features();
 		CFeatures* test=ui->ui_features->get_test_features();
-		if (test)
+		if (train && test)
 		{
 			EFeatureClass fclass=test->get_feature_class();
 			EFeatureType ftype=test->get_feature_type();
@@ -759,13 +798,23 @@ bool CGUIKernel::init_kernel(const char* target)
 			
 			{
 				if (!initialized)
-					SG_ERROR("Kernel not initialized with training examples.\n");
-				else
 				{
-					SG_INFO("Initialising kernel with TEST DATA, train: %p test %p\n", train, test);
-					// lhs -> always train_features; rhs -> always test_features
-					kernel->init(train, test);
+					EFeatureClass tr_fclass=train->get_feature_class();
+					EFeatureType tr_ftype=train->get_feature_type();
+					if ((k_fclass==tr_fclass || k_fclass==C_ANY || tr_fclass==C_ANY) &&
+							(k_ftype==tr_ftype || k_ftype==F_ANY || tr_ftype==F_ANY))
+					{
+						SG_INFO("Initialising kernel with TRAIN DATA, train: %p\n", train);
+						kernel->init(train, train);
+						initialized=true;
+					}
+					else
+						SG_ERROR("Kernel can not process this train feature type: %d %d.\n", fclass, ftype);
 				}
+				
+				SG_INFO("Initialising kernel with TEST DATA, train: %p test %p\n", train, test);
+				// lhs -> always train_features; rhs -> always test_features
+				kernel->init(train, test);
 			}
 			else
 				SG_ERROR("Kernel can not process this test feature type: %d %d.\n", fclass, ftype);
@@ -783,13 +832,19 @@ bool CGUIKernel::save_kernel(char* filename)
 {
 	if (kernel && initialized)
 	{
-		if (!kernel->save(filename))
-			SG_ERROR("Writing to file %s failed!\n", filename);
-		else
+		CAsciiFile* file=new CAsciiFile(filename);
+		try
 		{
-			SG_INFO("Successfully written kernel to \"%s\" !\n", filename);
-			return true;
+			kernel->save(file);
 		}
+		catch (...)
+		{
+			SG_ERROR("Writing to file %s failed!\n", filename);
+		}
+
+		SG_UNREF(file);
+		SG_INFO("Successfully written kernel to \"%s\" !\n", filename);
+		return true;
 	}
 	else
 		SG_ERROR("No kernel set / kernel not initialized!\n");
