@@ -22,6 +22,7 @@
 #include "GUIPreProc.h"
 #include "GUITime.h"
 #include "GUIStructure.h"
+#include "GUISignals.h"
 
 namespace shogun
 {
@@ -274,6 +275,8 @@ class CSGInterface : public CSGObject
 		bool cmd_set_svm_nu();
 		/** set SVM MKL parameters */
 		bool cmd_set_svm_mkl_parameters();
+		/** set ElasticnetMKL parameter lambda */
+		bool cmd_set_elasticnet_lambda();
 		/** set max train time */
 		bool cmd_set_max_train_time();
 		/** set SVM MKL enabled */
@@ -409,6 +412,38 @@ class CSGInterface : public CSGObject
 		 * precompute subkernels of a combined kernel
 		 */
 		bool cmd_precompute_subkernels();
+		/**
+		 * 
+		 */
+		bool cmd_signals_set_model() { return false; };
+		/**
+		 * 
+		 */
+		bool cmd_signals_set_positions();
+		/**
+		 * 
+		 */
+		bool cmd_signals_set_labels();
+		/**
+		 * 
+		 */
+		bool cmd_signals_set_split();
+		/**
+		 * 
+		 */
+		bool cmd_signals_set_train_mask();
+		/**
+		 * 
+		 */
+		bool cmd_signals_add_feature();
+		/**
+		 * 
+		 */
+		bool cmd_signals_add_kernel();
+		/**
+		 * 
+		 */
+		bool cmd_signals_run();
 		/** 
 		 * -precompute content svms 
 		 *  and save the outputs 
@@ -430,11 +465,20 @@ class CSGInterface : public CSGObject
 		 * -set lin feat
 		 */
 		bool cmd_set_lin_feat();
+
 		/** 
 		 * -init dyn prog
 		 */
 		bool cmd_init_dyn_prog();
 
+		/** 
+		 * clean up dyn prog
+		 */
+		bool cmd_clean_up_dyn_prog();
+
+		/**
+		 * initialize list of introns as DynProg features
+		 */
 		bool cmd_init_intron_list();
 
 		/** settings for long transition approximation*/
@@ -541,7 +585,6 @@ class CSGInterface : public CSGObject
 		virtual char* get_string(int32_t& len)=0;
 
 		virtual void get_bool_vector(bool*& vector, int32_t& len);
-		virtual void set_bool_vector(bool*& vector, int32_t& len);
 		virtual void get_byte_vector(uint8_t*& vector, int32_t& len)=0;
 		virtual void get_char_vector(char*& vector, int32_t& len)=0;
 		virtual void get_int_vector(int32_t*& vector, int32_t& len)=0;
@@ -620,6 +663,7 @@ class CSGInterface : public CSGObject
 		virtual void set_real(float64_t scalar)=0;
 		virtual void set_bool(bool scalar)=0;
 
+		virtual void set_bool_vector(const bool* vector, int32_t len);
 		virtual void set_byte_vector(const uint8_t* vector, int32_t len)=0;
 		virtual void set_char_vector(const char* vector, int32_t len)=0;
 		virtual void set_int_vector(const int32_t* vector, int32_t len)=0;
@@ -698,6 +742,7 @@ class CSGInterface : public CSGObject
 		CGUIPreProc* ui_preproc;
 		CGUITime* ui_time;
 		CGUIStructure* ui_structure;
+		//CGUISignals* ui_signals;
 
 	protected:
 		/* return true if str starts with cmd
