@@ -9,6 +9,20 @@ WORDSTRING_ORDER=3
 WORDSTRING_GAP=0
 WORDSTRING_REVERSE=False
 
+
+def get_features(fclass, ftype, data, *args, **kwargs):
+	if fclass=='simple':
+		return get_simple(ftype, data, *args, **kwargs)
+	elif fclass=='string':
+		return get_string(ftype, data, *args, **kwargs)
+	elif fclass=='string_complex':
+		return get_string_complex(ftype, data, *args, **kwargs)
+	elif fclass=='wd':
+		return get_wd(data, *args, **kwargs)
+	else:
+		raise ValueError, 'Unknown feature class %s.'%fclass
+
+
 def get_simple (ftype, data, alphabet=features.DNA, sparse=False):
 	"""Return SimpleFeatures.
 
@@ -40,6 +54,7 @@ def get_simple (ftype, data, alphabet=features.DNA, sparse=False):
 	else:
 		return {'train':train, 'test':test}
 
+
 def get_string (ftype, data, alphabet=features.DNA):
 	"""Return StringFeatures.
 
@@ -49,13 +64,11 @@ def get_string (ftype, data, alphabet=features.DNA):
 	@return Dict with StringFeatures train/test
 	"""
 
-	train=eval('features.String'+ftype+"Features(alphabet)")
-	train.set_string_features(data['train'])
-	test=eval('features.String'+ftype+"Features(alphabet)")
-	test.set_string_features(data['test'])
-
+	train=eval('features.String'+ftype+"Features(data['train'], alphabet)")
+	test=eval('features.String'+ftype+"Features(data['test'], alphabet)")
 	return {'train':train, 'test':test}
 
+
 def get_string_complex (ftype, data, alphabet=features.DNA,
 	order=WORDSTRING_ORDER, gap=WORDSTRING_GAP, reverse=WORDSTRING_REVERSE):
 	"""Return complex StringFeatures.
@@ -71,15 +84,13 @@ def get_string_complex (ftype, data, alphabet=features.DNA,
 
 	feats={}
 
-	charfeat=features.StringCharFeatures(alphabet)
-	charfeat.set_string_features(data['train'])
-	feat=eval('features.String'+ftype+'Features(charfeat.get_alphabet())')
+	charfeat=features.StringCharFeatures(data['train'], alphabet)
+	feat=eval('features.String'+ftype+'Features(alphabet)')
 	feat.obtain_from_char(charfeat, order-1, order, gap, reverse)
 	feats['train']=feat
 
-	charfeat=features.StringCharFeatures(alphabet)
-	charfeat.set_string_features(data['test'])
-	feat=eval('features.String'+ftype+'Features(charfeat.get_alphabet())')
+	charfeat=features.StringCharFeatures(data['test'], alphabet)
+	feat=eval('features.String'+ftype+'Features(alphabet)')
 	feat.obtain_from_char(charfeat, order-1, order, gap, reverse)
 	feats['test']=feat
 
@@ -89,6 +100,30 @@ def get_string_complex (ftype, data, alphabet=features.DNA,
 	else:
 		return feats
 
+
+def get_wd (data, order=WORDSTRING_ORDER):
+	"""Return WDFeatures.
+
+	@param data Train/test data for feature creation
+	@param order Order of the feature
+	@return Dict with WDFeatures train/test
+	"""
+
+	feats={}
+
+	charfeat=features.StringCharFeatures(data['train'], features.DNA)
+	bytefeat=features.StringByteFeatures(features.RAWDNA)
+	bytefeat.obtain_from_char(charfeat, 0, 1, 0, False)
+	feats['train']=features.WDFeatures(bytefeat, order, order)
+
+	charfeat=features.StringCharFeatures(data['test'], features.DNA)
+	bytefeat=features.StringByteFeatures(features.RAWDNA)
+	bytefeat.obtain_from_char(charfeat, 0, 1, 0, False)
+	feats['test']=features.WDFeatures(bytefeat, order, order)
+
+	return feats
+
+
 def add_preproc (name, feats, *args):
 	"""Add a preprocessor to the given features.
 
