@@ -5,22 +5,19 @@ distribution <- function(filename) {
 	sg('init_random', init_random)
 	set.seed(init_random)
 
-	if (!set_features()) {
+	if (!set_features('distribution_')) {
 		return(TRUE)
 	}
 
-	if (regexpr('^HMM', name)>0) {
+	if (regexpr('^HMM', distribution_name)>0) {
 		sg('new_hmm', distribution_N, distribution_M)
-		sg('bw')
+ 		sg('bw')
 	} else {
 		print('Cannot yet train other distributions than HMM!')
 		return(TRUE)
 	}
 
 	likelihood <- abs(sg('hmm_likelihood')-distribution_likelihood)
-	print(distribution_likelihood)
-	print(sg('hmm_likelihood'))
-
 	data <- list(likelihood, 0)
 	return(check_accuracy(distribution_accuracy, 'distribution', data))
 }
